import os
import platform
import re
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from test.integration.toolchains.apple.test_xcodetoolchain import _get_filename
from conan.test.utils.tools import TestClient

_expected_dep_xconfig = [
    "SYSTEM_HEADER_SEARCH_PATHS = $(inherited) $(SYSTEM_HEADER_SEARCH_PATHS_{name}_{name})",
    "GCC_PREPROCESSOR_DEFINITIONS = $(inherited) $(GCC_PREPROCESSOR_DEFINITIONS_{name}_{name})",
    "OTHER_CFLAGS = $(inherited) $(OTHER_CFLAGS_{name}_{name})",
    "OTHER_CPLUSPLUSFLAGS = $(inherited) $(OTHER_CPLUSPLUSFLAGS_{name}_{name})",
    "FRAMEWORK_SEARCH_PATHS = $(inherited) $(FRAMEWORK_SEARCH_PATHS_{name}_{name})",
    "LIBRARY_SEARCH_PATHS = $(inherited) $(LIBRARY_SEARCH_PATHS_{name}_{name})",
    "OTHER_LDFLAGS = $(inherited) $(OTHER_LDFLAGS_{name}_{name})",
]

_expected_conf_xconfig = [
    "SYSTEM_HEADER_SEARCH_PATHS_{name}_{name}[config={configuration}][arch={architecture}][sdk={sdk}{sdk_version}] = ",
    "GCC_PREPROCESSOR_DEFINITIONS_{name}_{name}[config={configuration}][arch={architecture}][sdk={sdk}{sdk_version}] = ",
    "OTHER_CFLAGS_{name}_{name}[config={configuration}][arch={architecture}][sdk={sdk}{sdk_version}] = ",
    "OTHER_CPLUSPLUSFLAGS_{name}_{name}[config={configuration}][arch={architecture}][sdk={sdk}{sdk_version}] = ",
    "FRAMEWORK_SEARCH_PATHS_{name}_{name}[config={configuration}][arch={architecture}][sdk={sdk}{sdk_version}] = ",
    "LIBRARY_SEARCH_PATHS_{name}_{name}[config={configuration}][arch={architecture}][sdk={sdk}{sdk_version}] = ",
    "OTHER_LDFLAGS_{name}_{name}[config={configuration}][arch={architecture}][sdk={sdk}{sdk_version}] = "
]


def expected_files(current_folder, configuration, architecture, sdk_version):
    files = []
    name = _get_filename(configuration, architecture, sdk_version)
    deps = ["hello", "goodbye"]
    files.extend(
        [os.path.join(current_folder, "conan_{dep}_{dep}{name}.xcconfig".format(dep=dep, name=name)) for dep in deps])
    files.append(os.path.join(current_folder, "conandeps.xcconfig"))
    return files


def check_contents(client, deps, configuration, architecture, sdk_version):
    for dep_name in deps:
        dep_xconfig = client.load("conan_{dep}_{dep}.xcconfig".format(dep=dep_name))
        conf_name = "conan_{}_{}{}.xcconfig".format(dep_name, dep_name,
                                                 _get_filename(configuration, architecture, sdk_version))

        assert '#include "{}"'.format(conf_name) in dep_xconfig
        for var in _expected_dep_xconfig:
            line = var.format(name=dep_name)
            assert line in dep_xconfig

        conan_conf = client.load(conf_name)
        for var in _expected_conf_xconfig:
            assert var.format(name=dep_name, configuration=configuration, architecture=architecture,
                              sdk="macosx", sdk_version=sdk_version) in conan_conf


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
def test_generator_files():
    client = TestClient()
    client.save({"hello.py": GenConanfile().with_settings("os", "arch", "compiler", "build_type")
                                           .with_package_info(cpp_info={"libs": ["hello"],
                                                                        "frameworks": ['framework_hello']},
                                                              env_info={})})
    client.run("export hello.py --name=hello --version=0.1")
    client.save({"goodbye.py": GenConanfile().with_settings("os", "arch", "compiler", "build_type")
                                             .with_package_info(cpp_info={"libs": ["goodbye"],
                                                                          "frameworks": ['framework_goodbye']},
                                                                env_info={})})
    client.run("export goodbye.py --name=goodbye --version=0.1")
    client.save({"conanfile.txt": "[requires]\nhello/0.1\ngoodbye/0.1\n"}, clean_first=True)

    for build_type in ["Release", "Debug"]:

        client.run("install . -g XcodeDeps -s build_type={} -s arch=x86_64 -s os.sdk_version=12.1 --build missing".format(build_type))

        for config_file in expected_files(client.current_folder, build_type, "x86_64", "12.1"):
            assert os.path.isfile(config_file)

        conandeps = client.load("conandeps.xcconfig")
        assert '#include "conan_hello.xcconfig"' in conandeps
        assert '#include "conan_goodbye.xcconfig"' in conandeps

        conan_config = client.load("conan_config.xcconfig")
        assert '#include "conandeps.xcconfig"' in conan_config

        check_contents(client, ["hello", "goodbye"], build_type, "x86_64", "12.1")


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
def test_generator_files_with_custom_config():
    client = TestClient()

    client.save({"hello.py": GenConanfile().with_settings("os", "arch", "compiler", "build_type")
                                           .with_package_info(cpp_info={"libs": ["hello"]},
                                                              env_info={})})
    client.run("export hello.py --name=hello --version=0.1")

    client.save({"goodbye.py": GenConanfile().with_settings("os", "arch", "compiler", "build_type")
                                             .with_package_info(cpp_info={"libs": ["goodbye"]},
                                                                env_info={})})
    client.run("export goodbye.py --name=goodbye --version=0.1")

    conanfile_py = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.apple import XcodeDeps
        class LibConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            options = {"XcodeConfigName": [None, "ANY"]}
            default_options = {"XcodeConfigName": None}
            requires = "hello/0.1", "goodbye/0.1"

            def generate(self):
                xcode = XcodeDeps(self)
                if self.options.get_safe("XcodeConfigName"):
                    xcode.configuration = str(self.options.get_safe("XcodeConfigName"))
                xcode.generate()
        """)

    client.save({"conanfile.py": conanfile_py})
    custom_config_name = "CustomConfig"

    for use_custom_config in [True, False]:
        for build_type in ["Release", "Debug"]:
            cli_command = "install . -s build_type={} -s arch=x86_64 -s os.sdk_version=12.1  --build missing".format(build_type)
            if use_custom_config:
                cli_command += " -o XcodeConfigName={}".format(custom_config_name)
                configuration_name = custom_config_name
            else:
                configuration_name = build_type

            client.run(cli_command)

            for config_file in expected_files(client.current_folder, configuration_name, "x86_64", "12.1"):
                assert os.path.isfile(config_file)

            conandeps = client.load("conandeps.xcconfig")
            assert '#include "conan_hello.xcconfig"' in conandeps
            assert '#include "conan_goodbye.xcconfig"' in conandeps

            conan_config = client.load("conan_config.xcconfig")
            assert '#include "conandeps.xcconfig"' in conan_config

            check_contents(client, ["hello", "goodbye"],  configuration_name, "x86_64", "12.1",)

@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
def test_xcodedeps_aggregate_components():
    client = TestClient()

    conanfile_py = textwrap.dedent("""
        from conan import ConanFile
        class LibConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            def package_info(self):
                self.cpp_info.includedirs = ["liba_include"]
        """)

    client.save({"conanfile.py": conanfile_py})

    client.run("create . --name=liba --version=1.0")

    r""""
        1   a
       / \ /
      2   3
       \ /
        4   5  6
        |   |  /
         \ / /
           7
    """

    conanfile_py = textwrap.dedent("""
        from conan import ConanFile
        class LibConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            requires = "liba/1.0"
            def package_info(self):
                self.cpp_info.components["libb_comp1"].includedirs = ["libb_comp1"]
                self.cpp_info.components["libb_comp1"].libdirs = ["mylibdir"]
                self.cpp_info.components["libb_comp2"].includedirs = ["libb_comp2"]
                self.cpp_info.components["libb_comp2"].libdirs = ["mylibdir"]
                self.cpp_info.components["libb_comp2"].requires = ["libb_comp1"]
                self.cpp_info.components["libb_comp3"].includedirs = ["libb_comp3"]
                self.cpp_info.components["libb_comp3"].libdirs = ["mylibdir"]
                self.cpp_info.components["libb_comp3"].requires = ["libb_comp1", "liba::liba"]
                self.cpp_info.components["libb_comp4"].includedirs = ["libb_comp4"]
                self.cpp_info.components["libb_comp4"].libdirs = ["mylibdir"]
                self.cpp_info.components["libb_comp4"].requires = ["libb_comp2", "libb_comp3"]
                self.cpp_info.components["libb_comp5"].includedirs = ["libb_comp5"]
                self.cpp_info.components["libb_comp5"].libdirs = ["mylibdir"]
                self.cpp_info.components["libb_comp6"].includedirs = ["libb_comp6"]
                self.cpp_info.components["libb_comp6"].libdirs = ["mylibdir"]
                self.cpp_info.components["libb_comp7"].includedirs = ["libb_comp7"]
                self.cpp_info.components["libb_comp7"].libdirs = ["mylibdir"]
                self.cpp_info.components["libb_comp7"].requires = ["libb_comp4", "libb_comp5", "libb_comp6"]
        """)

    client.save({"conanfile.py": conanfile_py})

    client.run("create . --name=libb --version=1.0")

    client.run("install --requires=libb/1.0 -g XcodeDeps")

    lib_entry = client.load("conan_libb.xcconfig")

    for index in range(1, 8):
        assert f"conan_libb_libb_comp{index}.xcconfig" in lib_entry

    component7_entry = client.load("conan_libb_libb_comp7.xcconfig")
    assert '#include "conan_liba.xcconfig"' in component7_entry

    arch_setting = client.get_default_host_profile().settings['arch']
    arch = "arm64" if arch_setting == "armv8" else arch_setting

    component7_vars = client.load(f"conan_libb_libb_comp7_release_{arch}.xcconfig")

    # all of the transitive required components and the component itself are added
    for index in range(1, 8):
        assert f"libb_comp{index}" in component7_vars

    assert "mylibdir" in component7_vars

    component4_vars = client.load(f"conan_libb_libb_comp4_release_{arch}.xcconfig")

    # all of the transitive required components and the component itself are added
    for index in range(1, 5):
        assert f"libb_comp{index}" in component4_vars

    for index in range(5, 8):
        assert f"libb_comp{index}" not in component4_vars

    # folders are aggregated
    assert "mylibdir" in component4_vars


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
def test_xcodedeps_traits():
    client = TestClient()
    conanfile_py = textwrap.dedent("""
        from conan import ConanFile
        class LibConan(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            {package_info}
            {requirements}
        """)

    package_info = """
    def package_info(self):
        self.cpp_info.components["cmp1"].includedirs = ["cmp1_includedir"]
        self.cpp_info.components["cmp2"].includedirs = ["cmp2_includedir"]

        self.cpp_info.components["cmp1"].libdirs = ["cmp1_libdir"]
        self.cpp_info.components["cmp2"].libdirs = ["cmp2_libdir"]
        self.cpp_info.components["cmp1"].libs = ["cmp1_lib"]
        self.cpp_info.components["cmp2"].libs = ["cmp2_lib"]
        self.cpp_info.components["cmp1"].system_libs = ["cmp1_system_lib"]
        self.cpp_info.components["cmp2"].system_libs = ["cmp2_system_lib"]
        self.cpp_info.components["cmp1"].frameworkdirs = ["cmp1_frameworkdir"]
        self.cpp_info.components["cmp2"].frameworkdirs = ["cmp2_frameworkdir"]
        self.cpp_info.components["cmp1"].frameworks = ["cmp1_framework"]
        self.cpp_info.components["cmp2"].frameworks = ["cmp2_framework"]

        self.cpp_info.components["cmp1"].defines = ["cmp1_define"]
        self.cpp_info.components["cmp2"].defines = ["cmp2_define"]
        self.cpp_info.components["cmp1"].cflags = ["cmp1_cflag"]
        self.cpp_info.components["cmp2"].cflags = ["cmp2_cflag"]
        self.cpp_info.components["cmp1"].cxxflags = ["cmp1_cxxflag"]
        self.cpp_info.components["cmp2"].cxxflags = ["cmp2_cxxflag"]
        self.cpp_info.components["cmp1"].sharedlinkflags = ["cmp1_sharedlinkflag"]
        self.cpp_info.components["cmp2"].sharedlinkflags = ["cmp2_sharedlinkflag"]
        self.cpp_info.components["cmp1"].exelinkflags = ["cmp1_exelinkflag"]
        self.cpp_info.components["cmp2"].exelinkflags = ["cmp2_exelinkflag"]
        """

    client.save({"lib_a.py": conanfile_py.format(requirements="", package_info=package_info)})

    client.run("create lib_a.py --name=lib_a --version=1.0")

    requirements = """
    def requirements(self):
        self.requires("lib_a/1.0", headers=False)
    """

    client.save({"lib_b.py": conanfile_py.format(requirements=requirements, package_info="")},
                clean_first=True)

    client.run("install lib_b.py -g XcodeDeps")

    arch_setting = client.get_default_host_profile().settings['arch']
    arch = "arm64" if arch_setting == "armv8" else arch_setting

    comp1_info = client.load(f"conan_lib_a_cmp1_release_{arch}.xcconfig")
    comp2_info = client.load(f"conan_lib_a_cmp2_release_{arch}.xcconfig")

    assert "cmp1_include" not in comp1_info
    assert "cmp2_include" not in comp2_info

    requirements = """
    def requirements(self):
        self.requires("lib_a/1.0", libs=False)
    """

    client.save({"lib_b.py": conanfile_py.format(requirements=requirements, package_info="")},
                clean_first=True)
    client.run("install lib_b.py -g XcodeDeps")

    comp1_info = client.load(f"conan_lib_a_cmp1_release_{arch}.xcconfig")
    comp2_info = client.load(f"conan_lib_a_cmp2_release_{arch}.xcconfig")

    assert "cmp1_frameworkdir" not in comp1_info
    assert "cmp2_frameworkdir" not in comp2_info

    assert "-lcmp1_lib -lcmp1_system_lib -framework cmp1_framework" not in comp1_info
    assert "-lcmp2_lib -lcmp2_system_lib -framework cmp2_framework" not in comp2_info

    requirements = """
    def requirements(self):
        self.requires("lib_a/1.0", headers=False, libs=False)
    """

    client.save({"lib_b.py": conanfile_py.format(requirements=requirements, package_info="")},
                clean_first=True)
    client.run("install lib_b.py -g XcodeDeps")

    # this changed from non-existing to existing after https://github.com/conan-io/conan/pull/15128
    existing = [f"conan_lib_a_cmp1_release_{arch}.xcconfig", "conan_lib_a_cmp1.xcconfig",
                    f"conan_lib_a_cmp2_release_{arch}.xcconfig", "conan_lib_a_cmp2.xcconfig",
                    "conan_lib_a.xcconfig"]

    for file in existing:
        assert os.path.exists(os.path.join(client.current_folder, file))

    assert '#include "conan_lib_a.xcconfig"' in client.load("conandeps.xcconfig")

    requirements = """
    def requirements(self):
        self.requires("lib_a/1.0", headers=False, libs=False, run=True)
    """

    client.save({"lib_b.py": conanfile_py.format(requirements=requirements, package_info="")},
                clean_first=True)

    client.run("install lib_b.py -g XcodeDeps")

    comp1_info = client.load(f"conan_lib_a_cmp1_release_{arch}.xcconfig")
    comp2_info = client.load(f"conan_lib_a_cmp2_release_{arch}.xcconfig")

    assert "cmp1_define" not in comp1_info
    assert "cmp2_define" not in comp2_info
    assert "cmp1_cflag" not in comp1_info
    assert "cmp2_cflag" not in comp2_info
    assert "cmp1_cxxflag" not in comp1_info
    assert "cmp2_cxxflag" not in comp2_info
    assert "cmp1_sharedlinkflag" not in comp1_info
    assert "cmp2_sharedlinkflag" not in comp2_info
    assert "cmp1_exelinkflag" not in comp1_info
    assert "cmp2_exelinkflag" not in comp2_info


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
def test_xcodedeps_frameworkdirs():
    client = TestClient()

    conanfile_py = textwrap.dedent("""
        from conan import ConanFile
        class LibConan(ConanFile):
            name = "lib_a"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            def package_info(self):
                self.cpp_info.frameworkdirs = ["lib_a_frameworkdir"]
        """)

    client.save({"conanfile.py": conanfile_py})
    client.run("create .")

    arch_setting = client.get_default_host_profile().settings['arch']
    arch = "arm64" if arch_setting == "armv8" else arch_setting

    client.run("install --requires=lib_a/1.0 -g XcodeDeps")

    lib_a_xcconfig = client.load(f"conan_lib_a_lib_a_release_{arch}.xcconfig")

    assert "lib_a_frameworkdir" in lib_a_xcconfig


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
def test_xcodedeps_cppinfo_requires():

    """
    lib_a: has four components cmp1, cmp2, cmp3, cmp4
    lib_b --> uses libA cmp1 so cpp_info.requires = ["lib_a::cmp1"]
    lib_c --> uses libA cmp2 so cpp_info.requires = ["lib_a::cmp2"]
    consumer --> libB, libC
    """
    client = TestClient()
    lib_a = textwrap.dedent("""
        from conan import ConanFile
        class lib_aConan(ConanFile):
            name = "lib_a"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            def package_info(self):
                self.cpp_info.components["cmp1"].includedirs = ["include"]
                self.cpp_info.components["cmp2"].includedirs = ["include"]
                self.cpp_info.components["cmp3"].includedirs = ["include"]
                self.cpp_info.components["cmp4"].includedirs = ["include"]
        """)

    lib = textwrap.dedent("""
        from conan import ConanFile
        class lib_{name}Conan(ConanFile):
            name = "lib_{name}"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            def requirements(self):
                self.requires("lib_a/1.0")
            def package_info(self):
                self.cpp_info.requires = {cppinfo_comps}
        """)

    consumer = textwrap.dedent("""
    from conan import ConanFile
    class ConsumerConan(ConanFile):
        name = "consumer"
        version = "1.0"
        settings = "os", "compiler", "build_type", "arch"
        generators = "XcodeDeps"
        def requirements(self):
            self.requires("lib_b/1.0")
            self.requires("lib_c/1.0")
    """)

    client.save({
        'lib_a/conanfile.py': lib_a,
        'lib_b/conanfile.py': lib.format(name="b", cppinfo_comps='["lib_a::cmp1"]'),
        'lib_c/conanfile.py': lib.format(name="c", cppinfo_comps='["lib_a::cmp2"]'),
        'consumer/conanfile.py': consumer,
    })

    client.run("create lib_a")

    client.run("create lib_b")

    client.run("create lib_c")

    client.run("install consumer")

    """
    Check that the generated lib_b and lib_c xcconfig only use the cmp1 and cmp2 components
    So we will only link against the components specified in the cpp_info.requires of lib_b and lib_c
    """

    lib_b = client.load(os.path.join("consumer", "conan_lib_b_lib_b.xcconfig"))

    # check that nothing from other components than the specified in the cpp_info.requires
    # from lib_b and lib_c exist in the xcconfig that adds the includes from components
    assert "cmp1" in lib_b
    assert "cmp2" not in lib_b
    assert "cmp3" not in lib_b
    assert "cmp4" not in lib_b

    lib_c = client.load(os.path.join("consumer", "conan_lib_c_lib_c.xcconfig"))

    assert "cmp1" not in lib_c
    assert "cmp2" in lib_c
    assert "cmp3" not in lib_c
    assert "cmp4" not in lib_c


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
def test_dependency_of_dependency_components():
    # testing: https://github.com/conan-io/conan/pull/11772

    """
    When a dependency of a dependency would have components, only the default
    name conan_dep_dep.xconfig would be included. However, this file was never
    generated, as they are in the form conan_dep_component.xconfig.

    lib_a -> lib_b -> lib_c (with components)
    """
    client = TestClient()
    lib_a = GenConanfile("lib_a", "1.0").with_require("lib_b/1.0").with_settings("os", "arch", "build_type", "compiler")
    lib_b = GenConanfile("lib_b", "1.0").with_require("lib_c/1.0").with_settings("os", "arch", "build_type", "compiler")

    lib_c = textwrap.dedent("""
        from conan import ConanFile
        class lib_aConan(ConanFile):
            name = "lib_c"
            version = "1.0"
            settings = "os", "compiler", "build_type", "arch"
            def package_info(self):
                self.cpp_info.components["cmp1"].includedirs = ["include_cmp1"]
                self.cpp_info.components["cmp2"].includedirs = ["include_cmp2"]
        """)

    client.save({
        'conanfile.py': lib_a,
        'lib_b/conanfile.py': lib_b,
        'lib_c/conanfile.py': lib_c,
    })

    client.run("create lib_c")

    client.run("create lib_b")

    client.run("install . -g XcodeDeps")

    lib_b_xconfig = client.load("conan_lib_b_lib_b.xcconfig")

    assert '#include "conan_lib_c_cmp1.xcconfig"' in lib_b_xconfig
    assert '#include "conan_lib_c_cmp1.xcconfig"' in lib_b_xconfig
    assert '#include "conan_lib_c_lib_c.xcconfig"' not in lib_b_xconfig


def test_skipped_not_included():
    # https://github.com/conan-io/conan/issues/13818
    client = TestClient()
    pkg_info = {"components": {"component": {"defines": ["SOMEDEFINE"]}}}

    client.save({"dep/conanfile.py": GenConanfile().with_package_type("header-library")
                                                   .with_package_info(cpp_info=pkg_info,
                                                                      env_info={}),
                 "pkg/conanfile.py": GenConanfile().with_requirement("dep/0.1")
                                                   .with_package_type("library")
                                                   .with_shared_option(),
                 "consumer/conanfile.py": GenConanfile().with_requires("pkg/0.1")
                                                        .with_settings("os", "build_type", "arch")})
    client.run("create dep --name=dep --version=0.1")
    client.run("create pkg --name=pkg --version=0.1")
    client.run("install consumer -g XcodeDeps -s arch=x86_64 -s build_type=Release")
    assert re.search(r"Skipped binaries\n\s+(.*?)", client.out, re.DOTALL)
    dep_xconfig = client.load("consumer/conan_pkg_pkg.xcconfig")
    assert "conan_dep.xcconfig" not in dep_xconfig


def test_correctly_handle_transitive_components():
    # https://github.com/conan-io/conan/issues/14887
    client = TestClient()
    has_components = textwrap.dedent("""
        from conan import ConanFile
        class PkgWithComponents(ConanFile):
            name = 'has_components'
            version = '1.0'
            settings = 'os', 'compiler', 'arch', 'build_type'
            def package_info(self):
                self.cpp_info.components['first'].libs = ['first']
                self.cpp_info.components['second'].libs = ['donottouch']
                self.cpp_info.components['second'].requires = ['first']
        """)

    uses_components = textwrap.dedent("""
        from conan import ConanFile
        class PkgUsesComponent(ConanFile):
            name = 'uses_components'
            version = '1.0'
            settings = 'os', 'compiler', 'arch', 'build_type'
            def requirements(self):
                self.requires('has_components/1.0')
            def package_info(self):
                self.cpp_info.libs = ['uses_only_first']
                self.cpp_info.requires = ['has_components::first']
        """)

    consumer = textwrap.dedent("""
        [requires]
        uses_components/1.0
        """)

    client.save({"has_components.py": has_components,
                 "uses_components.py": uses_components,
                 "consumer.txt": consumer})
    client.run("create has_components.py")
    client.run("create uses_components.py")
    client.run("install consumer.txt -g XcodeDeps")
    conandeps = client.load("conandeps.xcconfig")
    assert '#include "conan_has_components.xcconfig"' not in conandeps
    assert '#include "conan_uses_components.xcconfig"' in conandeps
    conan_uses_xcconfig = client.load("conan_uses_components_uses_components.xcconfig")
    assert '#include "conan_has_components_first.xcconfig"' in conan_uses_xcconfig
    assert '#include "conan_has_components_second.xcconfig"' not in conan_uses_xcconfig


def test_dont_add_skipped_xcconfigs_when_required_by_components():
    client = TestClient()
    regular_lib = textwrap.dedent("""
        from conan import ConanFile
        class PkgWithComponents(ConanFile):
            name = 'regular_lib'
            version = '1.0'
            settings = 'os', 'compiler', 'arch', 'build_type'
            def requirements(self):
                self.requires('header_skip/1.0')
                self.requires('header_transitive/1.0', transitive_headers=True)
            def package_info(self):
                self.cpp_info.components['component'].requires = ['header_skip::header_skip',
                                                                  'header_transitive::header_transitive']
        """)

    header_transitive = textwrap.dedent("""
        from conan import ConanFile
        class PkgUsesComponent(ConanFile):
            name = 'header_transitive'
            version = '1.0'
            settings = 'os', 'compiler', 'arch', 'build_type'
            package_type = 'header-library'
            def package_info(self):
                self.cpp_info.includedirs = ["include"]
        """)

    header_skip = textwrap.dedent("""
        from conan import ConanFile
        class PkgUsesComponent(ConanFile):
            name = 'header_skip'
            version = '1.0'
            settings = 'os', 'compiler', 'arch', 'build_type'
            package_type = 'header-library'
            def package_info(self):
                self.cpp_info.includedirs = ["include"]
        """)

    client.save({"header_transitive.py": header_transitive,
                 "header_skip.py": header_skip,
                 "regular_lib.py": regular_lib})
    client.run("create header_transitive.py")
    client.run("create header_skip.py")
    client.run("create regular_lib.py")
    client.run("install --requires=regular_lib/1.0 -g XcodeDeps")

    conandeps = client.load("conan_regular_lib_component.xcconfig")
    assert '#include "conan_header_skip.xcconfig"' not in conandeps
    assert '#include "conan_header_transitive.xcconfig"' in conandeps

    # Verify that header_skip xcconfig files are NOT generated (skipped dependency)
    skip_files = [f for f in os.listdir(client.current_folder) if 'header_skip' in f and f.endswith('.xcconfig')]
    assert len(skip_files) == 0, f"Header skip files should not be generated: {skip_files}"

    # Verify that header_transitive xcconfig files ARE generated (transitive dependency)
    transitive_files = [f for f in os.listdir(client.current_folder) if 'header_transitive' in f and f.endswith('.xcconfig')]
    assert len(transitive_files) > 0, f"Header transitive files should be generated: {transitive_files}"



import platform
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() == "Windows", reason="Needs pkg-config")
@pytest.mark.tool("pkg_config")
def test_pkgconfigdeps_definitions_escape():
    client = TestClient(path_with_spaces=False)
    conanfile = textwrap.dedent(r'''
        from conan import ConanFile
        class HelloLib(ConanFile):
            def package_info(self):
                self.cpp_info.defines.append("USER_CONFIG=\"user_config.h\"")
                self.cpp_info.defines.append('OTHER="other.h"')
                self.cpp_info.cflags.append("flag1=\"my flag1\"")
                self.cpp_info.cxxflags.append('flag2="my flag2"')
        ''')
    client.save({"conanfile.py": conanfile})
    client.run("export . --name=hello --version=1.0")
    client.save({"conanfile.txt": "[requires]\nhello/1.0\n"}, clean_first=True)
    client.run("install . --build=missing -g PkgConfigDeps")
    client.run_command("PKG_CONFIG_PATH=$(pwd) pkg-config --cflags hello")
    assert r'flag2=\"my flag2\" flag1=\"my flag1\" ' \
           r'-DUSER_CONFIG=\"user_config.h\" -DOTHER=\"other.h\"' in client.out


@pytest.mark.tool("cmake")
def test_pkgconfigdeps_with_test_requires():
    """
    PkgConfigDeps has to create any test requires declared on the recipe.

    Related issue: https://github.com/conan-io/conan/issues/11376
    """
    client = TestClient()
    client.save({"app/conanfile.py": GenConanfile("app", "1.0"),
                 "test/conanfile.py": GenConanfile("test", "1.0")})
    client.run("create app")
    client.run("create test")
    # Create library having build and test requires
    conanfile = textwrap.dedent(r'''
        from conan import ConanFile
        class HelloLib(ConanFile):
            def build_requirements(self):
                self.test_requires('app/1.0')
                self.test_requires('test/1.0')
        ''')
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("install . -g PkgConfigDeps")
    assert "Description: Conan package: test" in client.load("test.pc")
    assert "Description: Conan package: app" in client.load("app.pc")


@pytest.mark.skipif(platform.system() != "Windows", reason="It makes sense only for Windows")
@pytest.mark.tool("meson")  # https://github.com/mesonbuild/meson/pull/11649 is part of Meson 1.1.0
@pytest.mark.tool("pkg_config")
def test_pkgconfigdeps_bindir_and_meson():
    """
    This test checks that the field bindir introduced by PkgConfigDeps is useful for Windows
    OS and shared=True where all the DLL's files are located by default there.

    Basically, Meson (version >= 1.1.0) reads from the *.pc files the bindir variable if exists, and
    uses that variable to link with if SHARED libraries.

    Issue: https://github.com/conan-io/conan/issues/13532
    """
    client = TestClient()
    client.run("new meson_lib -d name=hello -d version=1.0")
    client.run("create . -tf \"\" -o *:shared=True")
    test_meson_build = textwrap.dedent("""
    project('Testhello', 'cpp')
    hello = dependency('hello', version : '>=1.0')
    example = executable('example', 'src/example.cpp', dependencies: hello)
    test('./src/example', example)
    """)
    test_conanfile = textwrap.dedent("""
    import os
    from conan import ConanFile
    from conan.tools.build import can_run
    from conan.tools.meson import Meson
    from conan.tools.layout import basic_layout

    class helloTestConan(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        generators = "PkgConfigDeps", "MesonToolchain"
        options = {"shared": [True, False], "fPIC": [True, False]}
        default_options = {"shared": True, "fPIC": True}

        def requirements(self):
            self.requires("hello/1.0")

        def build(self):
            meson = Meson(self)
            meson.configure()
            meson.build()

        def layout(self):
            basic_layout(self)
    """)
    client.save({
        "test_package/conanfile.py": test_conanfile,
        "test_package/meson.build": test_meson_build
    })
    client.run("build test_package/conanfile.py -o *:shared=True")
    # Important: Only Meson >= 1.1.0 brings this capability
    # Executing directly "meson test" fails if the bindir field does not exist
    client.run_command("meson test -C test_package/build-release")
    assert "1/1 ./src/example OK"


def test_pkgconfigdeps_component_matches_package_name():
    client = TestClient(path_with_spaces=False)
    # Create library having build and test requires
    conanfile = textwrap.dedent(r'''
        from conan import ConanFile
        class MyLib(ConanFile):
            name = "hello"
            version = "0.1"
            def package_info(self):
                self.cpp_info.components["mycomponent"].set_property("pkg_config_name", "hello")
        ''')
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("export-pkg .")
    client.run("install --requires=hello/0.1 -g PkgConfigDeps")
    content = client.load("hello.pc")
    assert "Conan component: hello" in content



import json
import os
import shutil
import sys
import textwrap

import pytest
import yaml

from conan.api.model import RecipeReference
from conan.test.utils.file_server import TestFileServer
from conan.test.utils.test_files import tgz_with_contents
from conan.test.utils.tools import TestClient, GenConanfile
from conan.internal.util.files import md5sum, sha1sum, sha256sum, load


class TestConanData:

    def test_conan_exports_kept(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Lib(ConanFile):
                exports = "myfile.txt"
            """)
        conandata = textwrap.dedent("""
            foo:
              bar: "as"
            """)
        client.save({"conanfile.py": conanfile,
                     "myfile.txt": "bar",
                     "conandata.yml": conandata})
        ref = RecipeReference.loads("lib/0.1")
        client.run(f"export . --name={ref.name} --version={ref.version}")
        export_folder = client.get_latest_ref_layout(ref).export()
        exported_data = os.path.join(export_folder, "conandata.yml")
        data = yaml.safe_load(load(exported_data))
        assert data == {"foo": {"bar": "as"}}
        assert os.path.exists(os.path.join(export_folder, "myfile.txt"))

    def test_conan_data_everywhere(self):
        client = TestClient(light=True)
        conanfile = """from conan import ConanFile

class Lib(ConanFile):

    def _assert_data(self):
        assert(self.conan_data["sources"]["all"]["url"] == "the url")
        assert(self.conan_data["sources"]["all"]["other"] == "field")
        self.output.info("My URL: {}".format(self.conan_data["sources"]["all"]["url"]))

    def configure(self):
        self._assert_data()

    def config_options(self):
        self._assert_data()

    def source(self):
        self._assert_data()

    def build(self):
        self._assert_data()

    def package(self):
        self._assert_data()

    def package_info(self):
        self._assert_data()
"""
        client.save({"conanfile.py": conanfile,
                     "conandata.yml": """
sources:
  all:
    url: "the url"
    other: "field"
"""})
        ref = RecipeReference.loads("lib/0.1")
        client.run(f"create . --name={ref.name} --version={ref.version}")
        assert "File 'conandata.yml' found. Exporting it..." in client.out
        assert "My URL:" in client.out
        export_folder = client.get_latest_ref_layout(ref).export()
        assert os.path.exists(os.path.join(export_folder, "conandata.yml"))

        # Transitive loaded?
        client.save({"conanfile.txt": "[requires]\n{}".format(ref)}, clean_first=True)
        client.run("install . ")
        assert "My URL:" in client.out
        client.run("install . --build='*'")
        assert "My URL:" in client.out

    def test_conan_data_as_source_newtools(self):
        client = TestClient(light=True)
        file_server = TestFileServer()
        client.servers["file_server"] = file_server

        tgz_path = tgz_with_contents({"foo.txt": "foo"})
        if sys.version_info.major == 3 and sys.version_info.minor >= 9:
            # Python 3.9 changed the tar algorithm. Conan tgz will have different checksums
            # https://github.com/conan-io/conan/issues/8020
            md5_value = "f1d0dee6f0bf5b7747c013dd26183cdb"
            sha1_value = "d45ca9ad171ca9baa93f4da99904036aa71b0ddb"
            sha256_value = "b6880ef494974b8413a107429bde8d6b81a85c45a600040f5334a1d300c203b5"
        else:
            md5_value = "babc50837f9aaf46e134455966230e3e"
            sha1_value = "1e5b8ff7ae58b40d698fe3d4da6ad2a47ec6f4f3"
            sha256_value = "3ff04581cb0e2f9e976a9baad036f4ca9d884907c3d9382bb42a8616d3c20e42"
        assert md5_value == md5sum(tgz_path)
        assert sha1_value == sha1sum(tgz_path)
        assert sha256_value == sha256sum(tgz_path)

        shutil.copy2(tgz_path, file_server.store)

        conanfile = textwrap.dedent("""
                from conan import ConanFile
                from conan.tools.files import get

                class Lib(ConanFile):
                    def source(self):
                        data = self.conan_data["sources"]["all"]
                        get(self, **data)
                        self.output.info("OK!")
                """)
        conandata = textwrap.dedent("""
                sources:
                  all:
                    url: "{}/myfile.tar.gz"
                    md5: "{}"
                    sha1: "{}"
                    sha256: "{}"
                """)
        client.save({"conanfile.py": conanfile,
                     "conandata.yml": conandata.format(file_server.fake_url, md5_value, sha1_value,
                                                       sha256_value)})

        client.run(f"create . --name=pkg --version=0.1")
        assert "OK!" in client.out

        ref_layout = client.exported_layout()
        source_folder = ref_layout.source()
        downloaded_file = os.path.join(source_folder, "foo.txt")
        assert "foo" == load(downloaded_file)

    def test_invalid_yml(self):
        client = TestClient(light=True)
        client.save({"conanfile.py": GenConanfile(),
                     "conandata.yml": ">>>> ::"})
        ref = RecipeReference.loads("lib/0.1")
        client.run(f"create . --name={ref.name} --version={ref.version}", assert_error=True)
        assert "ERROR: Error loading conanfile at" in client.out
        assert ": Invalid yml format at conandata.yml: while scanning a block scalar" in client.out

    def test_conan_data_development_flow(self):
        client = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile

            class Lib(ConanFile):
                def layout(self):
                    self.folders.build = "tmp/build"

                def _assert_data(self):
                    assert(self.conan_data["sources"]["all"]["url"] == "this url")
                    assert(self.conan_data["sources"]["all"]["other"] == "field")
                    self.output.info("My URL: {}".format(self.conan_data["sources"]["all"]["url"]))

                def source(self):
                    self._assert_data()

                def build(self):
                    self._assert_data()

                def package(self):
                    self._assert_data()
            """)
        conandata = textwrap.dedent("""
            sources:
              all:
                url: "this url"
                other: "field"
        """)
        client.save({"conanfile.py": conanfile,
                     "conandata.yml": conandata})
        client.run("source .")
        assert "My URL: this url" in client.out
        client.run("build . -of=tmp/build")
        assert "My URL: this url" in client.out
        client.run("export-pkg . --name=name --version=version")
        assert "My URL: this url" in client.out

    def test_conan_data_serialize(self):
        c = TestClient(light=True)
        conandata = textwrap.dedent("""
            src:
                url: "this url"
        """)
        c.save({"conanfile.py": GenConanfile("pkg", "0.1"),
                "conandata.yml": conandata})
        c.run("graph info . --format=json")
        graph = json.loads(c.stdout)
        assert graph["graph"]["nodes"]["0"]["conandata"]["src"] == {"url": "this url"}

        c.run("inspect . --format=json")
        result = json.loads(c.stdout)
        assert result["conandata"]["src"] == {"url": "this url"}


class TestConanDataUpdate:
    """
    testing the update_conandata() method
    """
    def test_conandata_update(self):
        """ test the update_conandata() helper
        """
        c = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import update_conandata
            class Pkg(ConanFile):
                name = "pkg"
                version = "0.1"
                def export(self):
                    update_conandata(self, {"sources": {"0.1": {"commit": 123, "type": "git"},
                                                        "0.2": {"url": "new"}
                                                       }
                                           })

                def source(self):
                    data = self.conan_data["sources"]
                    self.output.info("0.1-commit: {}!!".format(data["0.1"]["commit"]))
                    self.output.info("0.1-type: {}!!".format(data["0.1"]["type"]))
                    self.output.info("0.1-url: {}!!".format(data["0.1"]["url"]))
                    self.output.info("0.2-url: {}!!".format(data["0.2"]["url"]))
            """)
        conandata = textwrap.dedent("""\
            sources:
                "0.1":
                    url: myurl
                    commit: 234
            """)
        c.save({"conanfile.py": conanfile,
                "conandata.yml": conandata})
        c.run("create .")
        assert "pkg/0.1: 0.1-commit: 123!!" in c.out
        assert "pkg/0.1: 0.1-type: git!!" in c.out
        assert "pkg/0.1: 0.1-url: myurl!!" in c.out
        assert "pkg/0.1: 0.2-url: new!!" in c.out

    def test_conandata_update_error(self):
        """ test the update_conandata() helper fails if used outside export()
        """
        c = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import update_conandata
            class Pkg(ConanFile):
                name = "pkg"
                version = "0.1"
                def source(self):
                    update_conandata(self, {})
            """)
        c.save({"conanfile.py": conanfile})
        c.run("create .", assert_error=True)
        assert "The 'update_conandata()' can only be used in the 'export()' method" in c.out

    def test_conandata_create_if_not_exist(self):
        """ test the update_conandata() creates the file if it doesn't exist
        """
        c = TestClient(light=True)
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            from conan.tools.files import update_conandata
            class Pkg(ConanFile):
                name = "pkg"
                version = "0.1"
                def export(self):
                    update_conandata(self, {"data": "value"})
            """)
        c.save({"conanfile.py": conanfile})
        c.run("export .")  # It doesn't fail
        assert "pkg/0.1: Calling export()" in c.out


def test_conandata_trim():
    """ test the explict trim_conandata() helper
    """
    c = TestClient(light=True)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import trim_conandata

        class Pkg(ConanFile):
            name = "pkg"
            def export(self):
                trim_conandata(self)
        """)
    conandata_yml = textwrap.dedent("""\
        sources:
          "1.0":
            url: "url1"
            sha256: "sha1"
        patches:
          "1.0":
            - patch_file: "patches/some_patch"
              base_path: "source_subfolder"
        something: else
          """)
    c.save({"conanfile.py": conanfile,
            "conandata.yml": conandata_yml})
    c.run("export . --version=1.0")
    layout = c.exported_layout()
    data1 = load(os.path.join(layout.export(), "conandata.yml"))
    assert "pkg/1.0: Exported: pkg/1.0#70612e15e4fc9af1123fe11731ac214f" in c.out
    conandata_yml2 = textwrap.dedent("""\
        sources:
         "1.0":
           url: "url1"
           sha256: "sha1"
         "1.1":
           url: "url2"
           sha256: "sha2"
        patches:
         "1.1":
           - patch_file: "patches/some_patch2"
             base_path: "source_subfolder"
         "1.0":
           - patch_file: "patches/some_patch"
             base_path: "source_subfolder"
        something: else
        """)
    c.save({"conandata.yml": conandata_yml2})
    c.run("export . --version=1.0")
    layout = c.exported_layout()
    data2 = load(os.path.join(layout.export(), "conandata.yml"))
    assert "1.1" not in data2
    assert data1 == data2
    assert "pkg/1.0: Exported: pkg/1.0#70612e15e4fc9af1123fe11731ac214f" in c.out
    # If I now try to create version 1.2 which has no patches, and then change a patch
    # its revision should not change either
    conandata_yml3 = textwrap.dedent("""\
    sources:
      "1.0":
        url: "url1"
        sha256: "sha1"
      "1.1":
        url: "url2"
        sha256: "sha2"
      "1.3":
        url: "url3"
        sha256: "sha3"
    patches:
      "1.1":
        - patch_file: "patches/some_patch2"
          base_path: "source_subfolder"
      "1.0":
        - patch_file: "patches/some_patch"
          base_path: "source_subfolder"
    something: else""")
    c.save({"conandata.yml": conandata_yml3})
    c.run("export . --version=1.3")
    initial_v13_rev = c.exported_recipe_revision()
    conandata_yml4 = textwrap.dedent("""\
        sources:
          "1.0":
            url: "url1"
            sha256: "sha1"
          "1.1":
            url: "url2"
            sha256: "sha2"
          "1.3":
            url: "url3"
            sha256: "sha3"
        patches:
          "1.1":
            - patch_file: "patches/some_patch2-v2"
              base_path: "source_subfolder"
          "1.0":
            - patch_file: "patches/some_patch"
              base_path: "source_subfolder"
        something: else""")
    c.save({"conandata.yml": conandata_yml4})
    c.run("export . --version=1.3")
    second_v13_rev = c.exported_recipe_revision()
    assert initial_v13_rev == second_v13_rev


def test_trim_conandata_as_hook():
    c = TestClient(light=True)
    c.save_home({"extensions/hooks/hook_trim.py": textwrap.dedent("""
    from conan.tools.files import trim_conandata

    def post_export(conanfile):
        trim_conandata(conanfile)
    """)})

    conandata_yml = textwrap.dedent("""\
            sources:
              "1.0":
                url: "url1"
                sha256: "sha1"
            patches:
              "1.0":
                - patch_file: "patches/some_patch"
                  base_path: "source_subfolder"
            something: else
              """)
    c.save({"conanfile.py": GenConanfile("pkg"),
            "conandata.yml": conandata_yml})
    c.run("export . --version=1.0")
    layout = c.exported_layout()
    data1 = load(os.path.join(layout.export(), "conandata.yml"))
    assert "pkg/1.0: Exported: pkg/1.0#03af39add1c7c9d68dcdb10b6968a14d" in c.out
    conandata_yml2 = textwrap.dedent("""\
            sources:
             "1.0":
               url: "url1"
               sha256: "sha1"
             "1.1":
               url: "url2"
               sha256: "sha2"
            patches:
             "1.1":
               - patch_file: "patches/some_patch2"
                 base_path: "source_subfolder"
             "1.0":
               - patch_file: "patches/some_patch"
                 base_path: "source_subfolder"
            something: else
            """)
    c.save({"conandata.yml": conandata_yml2})
    c.run("export . --version=1.0")
    layout = c.exported_layout()
    data2 = load(os.path.join(layout.export(), "conandata.yml"))
    assert "1.1" not in data2
    assert data1 == data2
    assert "pkg/1.0: Exported: pkg/1.0#03af39add1c7c9d68dcdb10b6968a14d" in c.out


@pytest.mark.parametrize("raise_if_missing", [True, False])
def test_trim_conandata_as_hook_without_conandata(raise_if_missing):
    c = TestClient(light=True)
    c.save_home({"extensions/hooks/hook_trim.py": textwrap.dedent(f"""
    from conan.tools.files import trim_conandata

    def post_export(conanfile):
        trim_conandata(conanfile, raise_if_missing={raise_if_missing})
    """)})

    c.save({"conanfile.py": GenConanfile("pkg")})
    if raise_if_missing:
        c.run("export . --version=1.0", assert_error=True)
    else:
        c.run("export . --version=1.0")
        assert c.exported_recipe_revision() == "a9ec2e5fbb166568d4670a9cd1ef4b26"


def test_trim_conandata_anchors():
    """Anchors load correctly, because trim_conandata loads the yaml instead of replacing in place"""
    tc = TestClient(light=True)
    tc.save({"conanfile.py": textwrap.dedent("""
     from conan import ConanFile
     from conan.tools.files import trim_conandata

     class Pkg(ConanFile):
        name = "pkg"
        def export(self):
            trim_conandata(self)

        def generate(self):
            self.output.info("x: {}".format(self.conan_data["mapping"][self.version]["x"]))
    """),
             "conandata.yml": textwrap.dedent("""
             mapping:
                "1.0": &anchor
                    "x": "foo"
                "2.0": *anchor
             """)})
    tc.run("create . --version=2.0")
    assert "x: foo" in tc.out
    pkg_layout = tc.exported_layout()
    conandata = tc.load(pkg_layout.conandata())
    assert conandata == textwrap.dedent("""\
        mapping:
          '2.0':
            x: foo
        """)



import os
from pathlib import Path

from conan.internal.paths import get_conan_user_home, find_file_walk_up
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import chdir


class TestConanUserHome:
    def test_conanrc_abs_path_get_conan_user_home(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        folder_conan_runs = os.path.join(_temp_folder, "folder_where_conan_runs")
        os.mkdir(folder_conan_runs)
        with open(os.path.join(_temp_folder, ".conanrc"), 'w+') as file:
            file.write(f'conan_home={_temp_folder}\n')
        with chdir(folder_conan_runs):
            conan_home = get_conan_user_home()
            assert _temp_folder == conan_home


    def test_conanrc_local_path_get_conan_user_home(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        subfolder = "subfolder inside temp"
        with chdir(_temp_folder):
            with open(os.path.join(_temp_folder, ".conanrc"), 'w+') as file:
                file.write(f'conan_home=.{os.sep}{subfolder}\n')
            conan_home = get_conan_user_home()
            assert str(os.path.join(_temp_folder, subfolder)) == conan_home


    def test_conanrc_local_path_run_conan_subfolder_get_conan_user_home(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        folder_conan_runs = os.path.join(_temp_folder, "folder_where_conan_runs")
        os.mkdir(folder_conan_runs)
        with open(os.path.join(_temp_folder, ".conanrc"), 'w+') as file:
            file.write(f'conan_home=.{os.sep}\n')
        with chdir(folder_conan_runs):
            conan_home = get_conan_user_home()
            assert str(os.path.join(_temp_folder)) == conan_home


    def test_conanrc_local_outside_folder_path_get_conan_user_home(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        folder1 = os.path.join(_temp_folder, "folder1")
        os.mkdir(folder1)
        with chdir(folder1):
            with open(os.path.join(folder1, ".conanrc"), 'w+') as file:
                file.write(f'conan_home=..{os.sep}folder2\n')
            conan_home = get_conan_user_home()
            this_path = Path(_temp_folder) / "folder1" / f"..{os.sep}folder2"
            assert str(this_path) == str(conan_home)


    def test_conanrc_comments(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        with chdir(_temp_folder):
            with open(os.path.join(_temp_folder, ".conanrc"), 'w+') as file:
                file.write(f'#commenting something\nconan_home={_temp_folder}\n')
            conan_home = get_conan_user_home()
            assert _temp_folder == conan_home


    def test_conanrc_wrong_format(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        with chdir(_temp_folder):
            with open(os.path.join(_temp_folder, ".conanrc"), 'w+') as file:
                file.write(f'ronan_jome={_temp_folder}\n')
            conan_home = get_conan_user_home()
            assert _temp_folder not in conan_home


    def test_conanrc_not_existing(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        with chdir(_temp_folder):
            conan_home = get_conan_user_home()
            assert _temp_folder not in conan_home


class TestFindWalkFileUp:
    def test_find_file_walk_up(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        leaf = os.path.join(_temp_folder, "one", "two", "three")
        os.makedirs(leaf)
        file_name = "test.txt"
        file_path = os.path.join(leaf, file_name)
        with open(file_path, 'w+') as file:
            file.write("test")
        found_file = find_file_walk_up(leaf, file_name)
        assert str(found_file) == file_path

    def test_find_file_walk_up_no_file_end(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        leaf = os.path.join(_temp_folder, "one", "two", "three")
        os.makedirs(leaf)
        file_name = "test.txt"
        file_path = os.path.join(_temp_folder, file_name)
        with open(file_path, 'w+') as file:
            file.write("test")
        # We stop searching in "one", so we don't find the file one level above
        found_file = find_file_walk_up(leaf, file_name, end=os.path.join(_temp_folder, "one"))
        assert found_file is None

    def test_find_file_walk_up_no_file_no_end(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        leaf = os.path.join(_temp_folder, "one", "two", "three")
        os.makedirs(leaf)
        file_name = "test.txt"
        found_file = find_file_walk_up(leaf, file_name)
        assert found_file is None

    def test_find_file_walk_up_no_file_end_outside_leaf(self):
        _temp_folder = temp_folder(path_with_spaces=True)
        leaf = os.path.join(_temp_folder, "one", "two", "three")
        os.makedirs(leaf)
        file_name = "test.txt"
        _second_temp_folder = temp_folder(path_with_spaces=True)

        found_file = find_file_walk_up(leaf, file_name, end=_second_temp_folder)
        assert found_file is None



import os
import platform
import textwrap

import pytest

from conan.tools.build import cmd_args_to_string
from conan.test.utils.tools import TestClient
from conan.internal.util.files import chdir
from conan.internal.util.runners import detect_runner


@pytest.fixture(scope="module")
def application_folder():
    """
    This is building a simple C app to print the received arguments and see if the obtained
    value is the expected once it is escaped by the Conan cmd_args_to_string tool
    """
    t = TestClient()
    main = textwrap.dedent("""
    #include<stdio.h>
    int main(int argc, char *argv[]){
        int i;
        for(i=1;i<argc;i++){
            printf("%s\\n",argv[i]);
        }
        return 0;
    }
    """
    )

    cmake = textwrap.dedent("""
    cmake_minimum_required(VERSION 3.15)
    project(arg_printer C)

    add_executable(arg_printer main.c)
    """)

    t.save({"CMakeLists.txt": cmake, "main.c": main})
    t.run_command("cmake . -DCMAKE_BUILD_TYPE=Release")
    if platform.system() == "Windows":
        t.run_command("cmake --build . --config Release")
    else:
        t.run_command("cmake --build .")

    if platform.system() == "Windows":
        return os.path.join(t.current_folder, "Release")
    else:
        return t.current_folder


@pytest.mark.tool("cmake")
@pytest.mark.skipif(platform.system() == "Windows", reason="Unix console parsing")
@pytest.mark.parametrize("_input, output", [
         (['.', 'foo'], '. foo'),
         (['.', 'foo with sp'], ". 'foo with sp'"),
         (['.', 'foo "with" sp'], ". 'foo \"with\" sp'"),
         (['.', '\'foo with sp\''], '. \'\'"\'"\'foo with sp\'"\'"\'\''),
         (['"."', 'foo'], '\'"."\' foo'),
         (['".', 'foo'], '\'".\' foo'),
])
def test_unix_cases(application_folder, _input, output):
    _input = ["arg_printer"] + _input
    output = "arg_printer {}".format(output)
    assert cmd_args_to_string(_input) == output

    # Check calling the exe that the arguments are the same we tried to input
    with chdir(application_folder):
        _, cmd_parsed_output = detect_runner("./{}".format(output))
        interpreted_args = cmd_parsed_output.splitlines()
        real_args = _input[1:]
        assert real_args == interpreted_args


@pytest.mark.tool("cmake")
@pytest.mark.skipif(platform.system() != "Windows", reason="Windows console parsing")
@pytest.mark.parametrize("_input, output", [
         (['.', 'foo'], '. foo'),
         (['"."', '"foo"'], r'\".\" \"foo\"'),
         ([r'\".\"', '"foo"'], r'\\\".\\\" \"foo\"'),
         (['path with spaces/foo', 'var'], '"path with spaces/foo" var'),
         ([r'c:\path with spaces\foo', '%var$:.'], r'"c:\path with spaces\foo" %var$:.'),
])
def test_windows_cases(application_folder, _input, output):
    from subprocess import list2cmdline
    _input = ["arg_printer.exe"] + _input
    output = "arg_printer.exe {}".format(output)
    assert list2cmdline(_input) == output
    assert cmd_args_to_string(_input) == output

    # Check calling the exe that the arguments are the same we tried to input
    with chdir(application_folder):
        _, cmd_parsed_output = detect_runner(output)
        interpreted_args = cmd_parsed_output.splitlines()
        real_args = _input[1:]
        assert real_args == interpreted_args



import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_pkg_sign():
    c = TestClient(default_server_user=True)
    c.save({"conanfile.py": GenConanfile("pkg", "0.1").with_exports("export/*")
            .with_exports_sources("export_sources/*").with_package_file("myfile", "mycontents!"),
            "export/file1.txt": "file1!",
            "export_sources/file2.txt": "file2!"})
    signer = textwrap.dedent(r"""
        import os

        def sign(ref, artifacts_folder, signature_folder):
            print("Signing ref: ", ref)
            print("Signing folder: ", artifacts_folder)
            files = []
            for f in sorted(os.listdir(artifacts_folder)):
                if os.path.isfile(os.path.join(artifacts_folder, f)):
                    files.append(f)
            print("Signing files: ", sorted(files))
            signature = os.path.join(signature_folder, "signature.asc")
            open(signature, "w").write("\n".join(files))

        def verify(ref, artifacts_folder, signature_folder, files):
            print("Verifying ref: ", ref)
            print("Verifying folder: ", artifacts_folder)
            signature = os.path.join(signature_folder, "signature.asc")
            contents = open(signature).read()
            print("verifying contents", contents)
            for f in files:
                print("VERIFYING ", f)
                if os.path.isfile(os.path.join(artifacts_folder, f)):
                    assert f in contents
        """)
    c.save_home({"extensions/plugins/sign/sign.py": signer})
    c.run("create .")
    c.run("upload * -r=default -c")
    assert "Signing ref:  pkg/0.1" in c.out
    assert "Signing ref:  pkg/0.1:da39a3ee5e6b4b0d3255bfef95601890afd80709" in c.out
    # Make sure it is signing the sources too
    assert "Signing files:  ['conan_export.tgz', 'conan_sources.tgz', " \
           "'conanfile.py', 'conanmanifest.txt']" in c.out
    c.run("remove * -c")
    c.run("install --requires=pkg/0.1")
    assert "Verifying ref:  pkg/0.1" in c.out
    assert "Verifying ref:  pkg/0.1:da39a3ee5e6b4b0d3255bfef95601890afd80709" in c.out
    assert "VERIFYING  conanfile.py" in c.out
    assert "VERIFYING  conan_sources.tgz" not in c.out  # Sources not retrieved now
    # Lets force the retrieval of the sources
    c.run("install --requires=pkg/0.1 --build=*")
    assert "Verifying ref:  pkg/0.1" in c.out
    assert "VERIFYING  conanfile.py" not in c.out  # It doesn't re-verify previous contents
    assert "VERIFYING  conan_sources.tgz" in c.out



import os
import textwrap

import yaml
import pytest
from jinja2 import Template

from conan.internal.cache.cache import PkgCache
from conan.internal.cache.home_paths import HomePaths
from conan.internal.api.profile.profile_loader import ProfileLoader
from conan.internal.default_settings import default_settings_yml
from conan.errors import ConanException
from conan.internal.model.conf import ConfDefinition
from conan.internal.model.settings import Settings
from conan.test.utils.test_files import temp_folder
from conan.internal.util.files import save


class TestSettingsCppStd:

    @pytest.fixture(autouse=True)
    def setup(self):
        self.cache_folder = temp_folder()
        self.cache = PkgCache(self.cache_folder, ConfDefinition())
        self.home_paths = HomePaths(self.cache_folder)
        save(self.home_paths.profile_plugin_path, "")

    def _save_profile(self, compiler_cppstd=None, filename="default"):
        fullpath = os.path.join(self.home_paths.profiles_path, filename)

        t = Template(textwrap.dedent("""
            [settings]
            os=Macos
            arch=x86_64
            compiler=apple-clang
            {% if compiler_cppstd %}compiler.cppstd={{ compiler_cppstd }}{% endif %}
            compiler.libcxx=libc++
            compiler.version=10.0
            """))

        save(fullpath, t.render(compiler_cppstd=compiler_cppstd))
        return filename

    def test_no_compiler_cppstd(self):
        # https://github.com/conan-io/conan/issues/5128
        fullpath = os.path.join(self.home_paths.profiles_path, "default")
        t = textwrap.dedent("""
            [settings]
            os=Macos
            arch=x86_64
            compiler=apple-clang
            compiler.libcxx=libc++
            compiler.version=10.0
            compiler.cppstd = 14
            """)
        save(fullpath, t)
        profile_loader = ProfileLoader(self.cache_folder)
        with pytest.raises(ConanException,
                                    match="'settings.compiler.cppstd' doesn't exist for 'apple-clang'"):
            profile = profile_loader.from_cli_args(["default"], None, None, None, None)
            settings = Settings(yaml.safe_load(default_settings_yml.replace("cppstd", "foobar")))
            profile.process_settings(settings)

    def test_no_value(self):
        self._save_profile()
        profile_loader = ProfileLoader(self.cache_folder)
        r = profile_loader.from_cli_args(["default"], None, None, None, None)
        assert "compiler.cppstd" not in r.settings

    def test_value_none(self):
        self._save_profile(compiler_cppstd="None")
        profile_loader = ProfileLoader(self.cache_folder)
        # It is incorrect to assign compiler.cppstd=None in the profile
        with pytest.raises(ConanException, match="Invalid setting"):
            r = profile_loader.from_cli_args(["default"], None, None, None, None)
            settings = Settings(yaml.safe_load(default_settings_yml))
            r.process_settings(settings)

    def test_value_valid(self):
        self._save_profile(compiler_cppstd="11")
        profile_loader = ProfileLoader(self.cache_folder)
        r = profile_loader.from_cli_args(["default"], None, None, None, None)
        assert r.settings["compiler.cppstd"] == "11"
        assert "cppstd" not in r.settings

    def test_value_invalid(self):
        self._save_profile(compiler_cppstd="13")
        profile_loader = ProfileLoader(self.cache_folder)
        with pytest.raises(ConanException, match="Invalid setting '13' is not a valid "
                                                    "'settings.compiler.cppstd' value"):
            r = profile_loader.from_cli_args(["default"], None, None, None, None)
            settings = Settings(yaml.safe_load(default_settings_yml))
            r.process_settings(settings)



from conan.test.utils.tools import TestClient


class TestBuildTypeSetting:

    def test_build_type(self):
        # https://github.com/conan-io/conan/issues/2500
        client = TestClient()
        conanfile = """from conan import ConanFile
class Pkg(ConanFile):
    settings = "build_type"
    def build(self):
        self.output.info("BUILD TYPE: %s" % (self.settings.build_type or "Not defined"))
"""
        test_conanfile = """from conan import ConanFile
class Pkg(ConanFile):
    settings = "build_type"
    def requirements(self):
        self.requires(self.tested_reference_str)
    def build(self):
        self.output.info("BUILD TYPE: %s" % (self.settings.build_type or "Not defined"))
    def test(self):
        pass
"""
        client.save({"conanfile.py": conanfile,
                     "test_package/conanfile.py": test_conanfile,
                     "myprofile": ""})

        # This won't fail, as it has a build_type=None, which is allowed
        client.run("export . --name=pkg --version=0.1 --user=lasote --channel=testing")
        client.run("install --requires=pkg/0.1@lasote/testing -pr=myprofile --build='*'")
        assert 1 == str(client.out).count("BUILD TYPE: Not defined")

        # test_package is totally consinstent with the regular package
        client.run("create . --name=pkg --version=0.1 --user=lasote --channel=testing -pr=myprofile")
        assert 2 == str(client.out).count("BUILD TYPE: Not defined")

        client.save({"conanfile.py": conanfile,
                     "test_package/conanfile.py": test_conanfile,
                     "myprofile": "[settings]\nbuild_type=Release"})

        client.run("export . --name=pkg --version=0.1 --user=user --channel=testing")
        client.run("install --requires=pkg/0.1@lasote/testing -pr=myprofile --build='*'")
        assert 1 == str(client.out).count("BUILD TYPE: Release")

        client.run("create . --name=pkg --version=0.1 --user=lasote --channel=testing -pr=myprofile")
        assert 2 == str(client.out).count("BUILD TYPE: Release")

        # Explicit build_tyep=None is NOT allowed, it is not a valid value
        client.save({"conanfile.py": conanfile,
                     "test_package/conanfile.py": test_conanfile,
                     "myprofile": "[settings]\nbuild_type=None"})

        client.run("export . --name=pkg --version=0.1 --user=lasote --channel=testing")
        client.run("install --requires=pkg/0.1@lasote/testing -pr=myprofile --build='*'",
                   assert_error=True)
        assert "ERROR: Invalid setting 'None' is not a valid 'settings.build_type'" in client.out
        assert "Possible values are ['Debug', 'Release', 'RelWithDebInfo', 'MinSizeRel']" in \
            client.out



import textwrap

from conan.test.utils.tools import TestClient, GenConanfile


def test_transitive_editables_half_diamond():
    # https://github.com/conan-io/conan/issues/4445
    client = TestClient(light=True)
    client.save({"libc/conanfile.py": GenConanfile("libc", "0.1"),
                 "libb/conanfile.py": GenConanfile("libb", "0.1").with_require("libc/0.1"),
                 "liba/conanfile.py": GenConanfile("liba", "0.1").with_requires("libb/0.1",
                                                                                "libc/0.1")})
    client.run("editable add libc")
    client.run("create libb")
    client.run("install liba")
    assert "libc/0.1 - Editable" in client.out
    with client.chdir("liba/build"):
        client.run("install ..")
        assert "libc/0.1 - Editable" in client.out


def test_transitive_editable_test_requires():
    """
    This test was crashing because editable packages was "SKIP"ing some dependencies as
    "test_requires", but editables need the dependencies to generate() and to build()

    https://github.com/conan-io/conan/issues/13543
    """
    c = TestClient()
    pkga = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import CMakeDeps, cmake_layout

        class Pkg(ConanFile):
            name = "pkga"
            version = "1.0"

            # Binary configuration
            settings = "os", "compiler", "build_type", "arch"

            def build_requirements(self):
                self.test_requires("gtest/1.0")

            def layout(self):
                cmake_layout(self)

            def generate(self):
                cd = CMakeDeps(self)
                cd.generate()
        """)
    pkgb = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout

        class Pkg(ConanFile):
            name = "pkgb"
            version = "1.0"

            # Binary configuration
            settings = "os", "compiler", "build_type", "arch"

            def requirements(self):
                self.requires("pkga/1.0")

            def layout(self):
                cmake_layout(self)
            """)
    c.save({"gtest/conanfile.py": GenConanfile("gtest", "1.0"),
            "pkga/conanfile.py": pkga,
            "pkgb/conanfile.py": pkgb})
    c.run("create gtest")
    c.run("build pkga")
    c.run("editable add pkga")
    # This used to crash, due to paths in test_requires not being processed (package_info() not
    # being called
    c.run("build pkgb")


def test_transitive_editables_python_requires_version_range():
    # https://github.com/conan-io/conan/issues/14411
    client = TestClient(default_server_user=True, light=True)
    client.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
                 "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_python_requires("dep/[*]")})
    client.run("editable add pkg", assert_error=True)
    assert "Version range '*' from requirement 'dep/[*]' required by 'python_requires' " \
           "could not be resolved" in client.out
    client.run("export dep")
    client.run("upload * -r=default -c")
    client.run("remove * -c")
    client.run("editable add pkg")  # now it works, able to return from server if necessary
    assert "dep/0.1: Downloaded recipe revision" in client.out


def test_transitive_editables_build():
    # https://github.com/conan-io/conan/issues/6064
    c = TestClient()
    libb = textwrap.dedent("""\
        from conan import ConanFile
        class LibB(ConanFile):
            name = "libb"
            version = "0.1"
            build_policy = "missing"
            settings = "os", "compiler", "arch"

            def build_requirements(self):
                self.build_requires("liba/[>=0.0]")

            def requirements(self):
                self.requires("liba/[>=0.0]")
        """)
    c.save({"liba/conanfile.py": GenConanfile("liba", "0.1"),
            "libb/conanfile.py": libb,
            "app/conanfile.txt": "[requires]\nlibb/0.1"})
    c.run("editable add liba")
    c.run("editable add libb")
    c.run("install app --build=*")
    # It doesn't crash
    # Try also with 2 profiles
    c.run("install app -s:b os=Windows --build=*")
    # it doesn't crash


def test_transitive_editable_cascade_build():
    """
    https://github.com/conan-io/conan/issues/15292
    """
    c = TestClient(light=True)
    pkga = GenConanfile("pkga", "1.0").with_package_type("static-library")
    pkgb = GenConanfile("pkgb", "1.0").with_requires("pkga/1.0").with_package_type("static-library")
    pkgc = GenConanfile("pkgc", "1.0").with_requires("pkgb/1.0").with_package_type("static-library")
    app = GenConanfile("app", "1.0").with_requires("pkgc/1.0").with_package_type("application")
    c.save({"pkga/conanfile.py": pkga,
            "pkgb/conanfile.py": pkgb,
            "pkgc/conanfile.py": pkgc,
            "app/conanfile.py": app})
    c.run("create pkga")
    c.run("create pkgb")
    pkgb_id = c.created_package_id("pkgb/1.0")
    c.run("create pkgc")
    pkgc_id = c.created_package_id("pkgc/1.0")
    c.run("install app")
    # now we want to investigate pkga
    c.run("editable add pkga")
    pkga = GenConanfile("pkga", "1.0").with_package_type("static-library")\
                                      .with_class_attribute("some=42")
    c.save({"pkga/conanfile.py": pkga})
    c.run("install app")
    # The consumers didn't need a new binary, even if I modified pkg
    c.assert_listed_binary({"pkgb/1.0": (pkgb_id, "Cache"),
                            "pkgc/1.0": (pkgc_id, "Cache")})
    c.run("install app --build=editable")
    c.assert_listed_binary({"pkgb/1.0": (pkgb_id, "Cache"),
                            "pkgc/1.0": (pkgc_id, "Cache")})
    # But I can command perfectly what to do and when
    c.run("install app --build=editable --build=cascade")
    c.assert_listed_binary({"pkgb/1.0": (pkgb_id, "Build"),
                            "pkgc/1.0": (pkgc_id, "Build")})

    # Changes in pkga are good, we export it
    c.run("export pkga")
    # Now we remove the editable
    c.run("editable remove pkga")
    c.run("install app", assert_error=True)
    assert "ERROR: Missing prebuilt package for 'pkga/1.0'" in c.out


def test_transitive_editable_cascade_package_id():
    """
    https://github.com/conan-io/conan/issues/15292
    """
    c = TestClient(light=True)
    pkga = GenConanfile("pkga", "1.0").with_package_type("static-library")
    pkgb = GenConanfile("pkgb", "1.0").with_requires("pkga/1.0").with_package_type("static-library")
    pkgc = GenConanfile("pkgc", "1.0").with_requires("pkgb/1.0").with_package_type("shared-library")
    app = GenConanfile("app", "1.0").with_requires("pkgc/1.0").with_package_type("application")
    c.save({"pkga/conanfile.py": pkga,
            "pkgb/conanfile.py": pkgb,
            "pkgc/conanfile.py": pkgc,
            "app/conanfile.py": app})
    c.run("create pkga")
    c.run("create pkgb")
    pkgb_id = c.created_package_id("pkgb/1.0")
    c.run("create pkgc")
    c.run("install app")

    # now we want to investigate pkga, we put it in editable mode
    c.run("editable add pkga")
    pkga = GenConanfile("pkga", "1.0").with_package_type("static-library")\
                                      .with_class_attribute("some=42")
    c.save({"pkga/conanfile.py": pkga})
    c.run("install app", assert_error=True)
    assert "ERROR: Missing prebuilt package for 'pkgc/1.0'" in c.out
    c.run("install app --build=missing")
    pkgc_id = c.created_package_id("pkgc/1.0")
    # The consumers didn't need a new binary, even if I modified pkg
    c.assert_listed_binary({"pkgb/1.0": (pkgb_id, "Cache"),
                            "pkgc/1.0": (pkgc_id, "Build")})

    # Changes in pkga are good, we export it
    c.run("export pkga")
    # Now we remove the editable
    c.run("editable remove pkga")
    c.run("install app", assert_error=True)
    assert "ERROR: Missing prebuilt package for 'pkgc/1.0'" in c.out
    c.run("install app --build=missing")
    pkgc_id = c.created_package_id("pkgc/1.0")
    # The consumers didn't need a new binary, even if I modified pkg
    c.assert_listed_binary({"pkgb/1.0": (pkgb_id, "Cache"),
                            "pkgc/1.0": (pkgc_id, "Build")})


def test_warning_from_cache():
    c = TestClient(light=True)
    c.save({"pkga/conanfile.py":  GenConanfile("pkga", "1.0"),
            "pkgb/conanfile.py": GenConanfile("pkgb", "1.0").with_requires("pkga/1.0"),
            "pkgc/conanfile.py": GenConanfile("pkgc", "1.0").with_requires("pkgb/1.0")})
    c.run("editable add pkga")
    c.run("create pkgb")
    assert "pkgb/1.0: WARN: risk: Package is being built in the cache using editable" in c.out
    c.run("create pkgc")
    assert "pkgc/1.0: WARN: risk: Package is being built in the cache using editable" in c.out
    c.run("create pkgc --build=*")
    assert "pkgb/1.0: WARN: risk: Package is being built in the cache using editable" in c.out
    assert "pkgc/1.0: WARN: risk: Package is being built in the cache using editable" in c.out



from conan.test.assets.genconanfile import GenConanfile
from test.integration.graph.core.graph_manager_base import GraphManagerTest
from test.integration.graph.core.graph_manager_test import _check_transitive
from conan.test.utils.tools import TestClient


class TestAlias(GraphManagerTest):

    def test_basic(self):
        # app -> liba/latest -(alias)-> liba/0.1
        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")

        consumer = self.recipe_consumer("app/0.1", ["liba/(latest)"])
        deps_graph = self.build_consumer(consumer)

        assert 2 == len(deps_graph.nodes)
        app = deps_graph.root
        liba = app.edges[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[app])
        self._check_node(app, "app/0.1", deps=[liba])

    def test_alias_diamond(self):
        # app -> ----------------------------------> liba/0.1
        #   \ -> libb/0.1 -> liba/latest -(alias) ----->/
        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")
        self.recipe_cache("libb/0.1", requires=["liba/(latest)"])
        consumer = self.recipe_consumer("app/0.1", ["liba/0.1", "libb/0.1"])
        deps_graph = self.build_consumer(consumer)

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        liba = app.edges[0].dst
        libb = app.edges[1].dst

        self._check_node(liba, "liba/0.1#123", dependents=[app, libb])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app])
        self._check_node(app, "app/0.1", deps=[liba, libb])

        # node, include, link, build, run
        # It seems they are in link order
        _check_transitive(app, [(libb, True, True, False, False),
                                (liba, True, True, False, False)])
        _check_transitive(libb, [(liba, True, True, False, False)])

    def test_two_alias_diamond(self):
        # https://github.com/conan-io/conan/issues/3353
        # app -> liba/latest -(alias) --------------------------------------> liba/0.1
        #   \ -> libb/latest -(alias) -> libb/0.1 -> liba/latest -(alias) ----->/

        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")
        self.recipe_cache("libb/0.1", requires=["liba/(latest)"])
        self.alias_cache("libb/latest", "libb/0.1")

        consumer = self.recipe_consumer("app/0.1", ["liba/(latest)", "libb/(latest)"])
        deps_graph = self.build_consumer(consumer)

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        liba = app.edges[0].dst
        libb = app.edges[1].dst

        self._check_node(liba, "liba/0.1#123", dependents=[app, libb])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app])
        self._check_node(app, "app/0.1", deps=[liba, libb])

        # node, include, link, build, run
        # Seems reverse order
        _check_transitive(app, [(libb, True, True, False, False),
                                (liba, True, True, False, False)])
        _check_transitive(libb, [(liba, True, True, False, False)])

    def test_full_two_branches_diamond(self):
        # https://github.com/conan-io/conan/issues/3353
        # app -> libb/latest -(alias) -> libb/0.1 -> liba/latest -(alias) ----> liba/0.1
        #   \ -> libc/latest -(alias) -> libc/0.1 -> liba/latest -(alias) ----->/

        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")
        self.recipe_cache("libb/0.1", requires=["liba/(latest)"])
        self.recipe_cache("libc/0.1", requires=["liba/(latest)"])
        self.alias_cache("libb/latest", "libb/0.1")
        self.alias_cache("libc/latest", "libc/0.1")

        consumer = self.recipe_consumer("app/0.1", ["libb/(latest)", "libc/(latest)"])
        deps_graph = self.build_consumer(consumer)

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        libc = app.edges[1].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[libb, libc])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app])
        self._check_node(libc, "libc/0.1#123", deps=[liba], dependents=[app])
        self._check_node(app, "app/0.1", deps=[libb, libc])

        # node, include, link, build, run
        _check_transitive(app, [(libb, True, True, False, False),
                                (libc, True, True, False, False),
                                (liba, True, True, False, False)])
        _check_transitive(libb, [(liba, True, True, False, False)])

    def test_alias_bug(self):
        # https://github.com/conan-io/conan/issues/2252
        # app -> libb/0.1 -> liba/latest -(alias)->liba/0.1
        #   \ -> libc/0.1 ----/
        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")
        self.recipe_cache("libb/0.1", requires=["liba/(latest)"])
        self.recipe_cache("libc/0.1", requires=["liba/(latest)"])

        consumer = self.recipe_consumer("app/0.1", ["libb/0.1", "libc/0.1"])
        deps_graph = self.build_consumer(consumer)

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        libc = app.edges[1].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[libb, libc])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app])
        self._check_node(libc, "libc/0.1#123", deps=[liba], dependents=[app])
        self._check_node(app, "app/0.1", deps=[libb, libc])

        # node, include, link, build, run
        _check_transitive(app, [(libb, True, True, False, False),
                                (libc, True, True, False, False),
                                (liba, True, True, False, False)])
        _check_transitive(libb, [(liba, True, True, False, False)])

    def test_alias_tansitive(self):
        # app -> liba/giga -(alias)->-> liba/mega -(alias)-> liba/latest -(alias)->liba/0.1

        self.recipe_cache("liba/0.1")
        self.alias_cache("liba/latest", "liba/0.1")
        self.alias_cache("liba/mega", "liba/(latest)")
        self.alias_cache("liba/giga", "liba/(mega)")

        consumer = self.recipe_consumer("app/0.1", ["liba/(giga)"])
        deps_graph = self.build_consumer(consumer)

        assert 2 == len(deps_graph.nodes)
        app = deps_graph.root
        liba = app.edges[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[app])
        self._check_node(app, "app/0.1", deps=[liba])

        # node, include, link, build, run
        _check_transitive(app, [(liba, True, True, False, False)])


class TestAliasBuildRequires(GraphManagerTest):

    def test_non_conflicting_alias(self):
        # https://github.com/conan-io/conan/issues/5468
        # libc ----> libb -------------------> liba/0.1
        #   \-(build)-> liba/latest -(alias)-> liba/0.2
        self.recipe_cache("liba/0.1")
        self.recipe_cache("liba/0.2")
        self.alias_cache("liba/latest", "liba/0.2")
        self.recipe_cache("libb/0.1", ["liba/0.1"])
        consumer = self.recipe_consumer("app/0.1", ["libb/0.1"], build_requires=["liba/(latest)"])

        deps_graph = self.build_consumer(consumer)

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        liba_build = app.edges[1].dst
        liba = libb.edges[0].dst

        self._check_node(liba, "liba/0.1#123", dependents=[libb])
        self._check_node(liba_build, "liba/0.2#123", dependents=[app])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[app])
        self._check_node(app, "app/0.1", deps=[libb, liba_build])


def test_mixing_aliases_and_fix_versions():
    # cd/1.0 -----------------------------> cb/latest -(alias)-> cb/1.0 -> ca/1.0
    #   \-----> cc/latest -(alias)-> cc/1.0 ->/                             /
    #                                    \------ ca/latest -(alias)------->/
    client = TestClient(light=True)

    client.save({"conanfile.py": GenConanfile("ca", "1.0")})
    client.run("create . ")
    client.alias("ca/latest@",  "ca/1.0@")

    client.save({"conanfile.py": GenConanfile("cb", "1.0").with_requirement("ca/1.0@")})
    client.run("create . --name=cb --version=1.0")
    client.alias("cb/latest@",  "cb/1.0@")

    client.save({"conanfile.py": GenConanfile("cc", "1.0")
                .with_requirement("cb/(latest)")
                .with_requirement("ca/(latest)")})
    client.run("create . ")
    client.alias("cc/latest@",  "cc/1.0@")

    client.save({"conanfile.py": GenConanfile("cd", "1.0")
                .with_requirement("cb/(latest)")
                .with_requirement("cc/(latest)")})
    client.run("create . ")



import os
import textwrap

from conan.internal.cache.home_paths import HomePaths
from conan.test.utils.tools import TestClient


class TestErrorsProfilePlugin:
    """ when the plugin fails, we want a clear message and a helpful trace
    """
    def test_error_profile_plugin(self):
        c = TestClient()
        profile_plugin = textwrap.dedent("""\
            def profile_plugin(profile):
                settings = profile.kk
            """)
        c.save_home({"extensions/plugins/profile.py": profile_plugin})

        c.run("install --requires=zlib/1.2.3", assert_error=True)
        assert "Error while processing 'profile.py' plugin, line 2" in c.out
        assert "settings = profile.kk" in c.out

    def test_remove_plugin_file(self):
        c = TestClient()
        c.run("version")  # to trigger the creation
        os.remove(HomePaths(c.cache_folder).profile_plugin_path)
        c.run("profile show", assert_error=True)
        assert "ERROR: The 'profile.py' plugin file doesn't exist" in c.out

    def test_regresion_29(self):
        # https://github.com/conan-io/conan/issues/17247
        c = TestClient()
        c.save({"conanfile.txt": ""})
        c.run("install . -s compiler=clang -s compiler.version=19 -s compiler.cppstd=26")
        # doesn't fail anymore
        c.run("install . -s compiler=apple-clang -s compiler.version=16 -s compiler.cppstd=26")
        # doesn't fail anymore
        c.run("install . -s compiler=gcc -s compiler.version=14 -s compiler.cppstd=26")
        # doesn't fail anymore


def test_android_ndk_version():
    c = TestClient()
    c.run("profile show -s os=Android")
    assert "os.ndk_version" not in c.out
    c.run("profile show -s os=Android -s os.ndk_version=r26")
    assert "os.ndk_version=r26" in c.out
    c.run("profile show -s os=Android -s os.ndk_version=r26a")
    assert "os.ndk_version=r26a" in c.out



import os

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_same_pref_removal():
    c = TestClient()
    c.save({"conanfile.py": GenConanfile("pkg", "0.1")})
    c.run("export .")
    c.run("install --requires=pkg/0.1 --build=pkg*")
    layout = c.created_layout()
    pkg_folder1 = layout.package()
    assert os.path.exists(os.path.join(pkg_folder1, "conanmanifest.txt"))
    assert os.path.exists(os.path.join(pkg_folder1, "conaninfo.txt"))
    c.run("install --requires=pkg/0.1 --build=pkg*")
    layout = c.created_layout()
    pkg_folder2 = layout.package()
    assert pkg_folder1 == pkg_folder2
    assert os.path.exists(os.path.join(pkg_folder1, "conanmanifest.txt"))
    assert os.path.exists(os.path.join(pkg_folder1, "conaninfo.txt"))



import os
import platform
import textwrap
import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient

@pytest.mark.parametrize("basic_layout, expected_path", [
    ('basic_layout(self)', 'build-release'),
    ('basic_layout(self, build_folder="custom_build_folder")', 'custom_build_folder')])
def test_basic_layout_subproject(basic_layout, expected_path):
    c = TestClient()
    conanfile = textwrap.dedent(f"""
        from conan import ConanFile
        from conan.tools.layout import basic_layout
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "AutotoolsToolchain"
            def layout(self):
                self.folders.root = ".."
                self.folders.subproject = "pkg"
                {basic_layout}
        """)
    c.save({"pkg/conanfile.py": conanfile})
    c.run("install pkg")
    ext = "sh" if platform.system() != "Windows" else "bat"
    assert os.path.isfile(os.path.join(c.current_folder, "pkg", expected_path, "conan",
                                       "conanautotoolstoolchain.{}".format(ext)))


def test_editable_includes():
    c = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.layout import basic_layout

        class Dep(ConanFile):
            name = "dep"
            version = "0.1"
            def layout(self):
                basic_layout(self, src_folder="src")
            """)
    c.save({"dep/conanfile.py": dep,
            "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_settings("build_type", "arch")
                                                          .with_requires("dep/0.1")})
    c.run("editable add dep")
    c.run("install pkg -s arch=x86_64 -g CMakeDeps -g PkgConfigDeps -g XcodeDeps")
    data = c.load(f"pkg/dep-release-x86_64-data.cmake")
    assert 'set(dep_INCLUDE_DIRS_RELEASE "${dep_PACKAGE_FOLDER_RELEASE}/src/include")' in data
    pc = c.load("pkg/dep.pc")
    assert "includedir=${prefix}/src/include" in pc
    xcode = c.load("pkg/conan_dep_dep_release_x86_64.xcconfig")
    dep_path = os.path.join(c.current_folder, "dep", "src", "include")
    assert f"SYSTEM_HEADER_SEARCH_PATHS_dep_dep[config=Release][arch=x86_64][sdk=*] = \"{dep_path}\"" in xcode


def test_editable_includes_previously_defined():
    # if someone already defined self.cpp.source.includedirs make sure we don't overwrite
    # their value
    c = TestClient()
    dep = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.layout import basic_layout

        class Dep(ConanFile):
            name = "dep"
            version = "0.1"
            def layout(self):
                self.cpp.source.includedirs = ["somefolder"]
                basic_layout(self, src_folder="src")
            """)
    c.save({"dep/conanfile.py": dep,
            "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_settings("build_type", "arch")
                                                          .with_requires("dep/0.1")})
    c.run("editable add dep")
    c.run("install pkg -s arch=x86_64 -g CMakeDeps -g PkgConfigDeps -g XcodeDeps")
    data = c.load(f"pkg/dep-release-x86_64-data.cmake")
    assert 'set(dep_INCLUDE_DIRS_RELEASE "${dep_PACKAGE_FOLDER_RELEASE}/src/somefolder")' in data
    pc = c.load("pkg/dep.pc")
    assert "includedir=${prefix}/src/somefolder" in pc
    xcode = c.load("pkg/conan_dep_dep_release_x86_64.xcconfig")
    dep_path = os.path.join(c.current_folder, "dep", "src", "somefolder")
    assert f"SYSTEM_HEADER_SEARCH_PATHS_dep_dep[config=Release][arch=x86_64][sdk=*] = \"{dep_path}\"" in xcode



import os


def basic_layout(conanfile, src_folder=".", build_folder=None):
    subproject = conanfile.folders.subproject

    conanfile.folders.source = src_folder if not subproject else os.path.join(subproject, src_folder)
    if build_folder:
        conanfile.folders.build = build_folder if not subproject else os.path.join(subproject, build_folder)
    else:
        conanfile.folders.build = "build" if not subproject else os.path.join(subproject, "build")
        if conanfile.settings.get_safe("build_type"):
            conanfile.folders.build += "-{}".format(str(conanfile.settings.build_type).lower())
    conanfile.folders.generators = os.path.join(conanfile.folders.build, "conan")
    conanfile.cpp.build.bindirs = ["."]
    conanfile.cpp.build.libdirs = ["."]

    if not conanfile.cpp.source.includedirs:
        conanfile.cpp.source.includedirs = ["include"]



import os
import platform

import pytest

from conan.errors import ConanMigrationError
from conans.migrations import Migrator
from conan.test.utils.test_files import temp_folder


class FakeMigrator(Migrator):

    def __init__(self, cache_folder, current_version):
        self.cache_folder = cache_folder
        super(FakeMigrator, self).__init__(cache_folder, current_version)


class TestMigratorPermissionTest:

    @pytest.mark.skipif(platform.system() == "Windows", reason="Can't apply chmod on Windows")
    def test_invalid_permission(self):
        conf_path = temp_folder(False)
        os.chmod(conf_path, 0o444)
        conf_path = os.path.join(conf_path, "foo")
        migrator = FakeMigrator(conf_path, "latest")
        with pytest.raises(ConanMigrationError) as error:
            migrator.migrate()
        assert f"Can't write version file in '{conf_path}/version.txt'" in str(error.value)



import os
import time
from multiprocessing.pool import ThreadPool
from typing import List

from conan.api.model import PackagesList, Remote
from conan.api.output import ConanOutput
from conan.internal.api.upload import add_urls
from conan.internal.conan_app import ConanApp
from conan.internal.api.uploader import PackagePreparator, UploadExecutor, UploadUpstreamChecker, \
    gather_metadata
from conan.internal.rest.pkg_sign import PkgSignaturesPlugin
from conan.internal.rest.file_uploader import FileUploader
from conan.internal.errors import AuthenticationException, ForbiddenException
from conan.errors import ConanException


class UploadAPI:
    """ This API is used to upload recipes and packages to a remote server."""

    def __init__(self, conan_api, api_helpers):
        self._conan_api = conan_api
        self._api_helpers = api_helpers

    def check_upstream(self, package_list: PackagesList, remote: Remote,
                       enabled_remotes: List[Remote], force=False):
        """ Checks ``remote`` for the existence of the recipes and packages in ``package_list``.
        Items that are not present in the remote will add an ``upload`` key to the entry
        with the value ``True``.

        If the recipe has an upload policy of ``skip``, it will be discarded from the upload list.

        :parameter package_list: A ``PackagesList`` object with the recipes and packages to check.
        :parameter remote: Remote to check.
        :parameter enabled_remotes: List of enabled remotes. This is used to possibly load
            python_requires from the listed recipes if necessary.
        :parameter force: If ``True``, it will skip the check and mark that all items need to be
            uploaded. A ``force_upload`` key will be added to the entries that will be uploaded.
        """
        app = ConanApp(self._conan_api)
        for ref, _ in package_list.items():
            layout = app.cache.recipe_layout(ref)
            conanfile_path = layout.conanfile()
            conanfile = app.loader.load_basic(conanfile_path, remotes=enabled_remotes)
            if conanfile.upload_policy == "skip":
                ConanOutput().info(f"{ref}: Skipping upload of binaries, "
                                   "because upload_policy='skip'")
                package_list.recipe_dict(ref)["packages"] = {}

        UploadUpstreamChecker(app).check(package_list, remote, force)

    def prepare(self, package_list: PackagesList, enabled_remotes: List[Remote],
                metadata: List[str] = None):
        """Compress the recipes and packages and fill the upload_data objects
        with the complete information. It doesn't perform the upload nor checks upstream to see
        if the recipe is still there

        :param package_list: A PackagesList object with the recipes and packages to upload.
        :param enabled_remotes: A list of remotes that are enabled in the client.
            Recipe sources will attempt to be fetched from these remotes.
        :param metadata: A list of patterns of metadata that should be uploaded.
            Default ``None`` means all metadata will be uploaded together with the package artifacts.
            If metadata contains an empty string (``""``),
            it means that no metadata files should be uploaded."""
        if metadata and metadata != [''] and '' in metadata:
            raise ConanException("Empty string and patterns can not be mixed for metadata.")
        app = ConanApp(self._conan_api)
        preparator = PackagePreparator(app, self._api_helpers.global_conf)
        preparator.prepare(package_list, enabled_remotes)
        if metadata != ['']:
            gather_metadata(package_list, app.cache, metadata)
        signer = PkgSignaturesPlugin(app.cache, app.cache_folder)
        # This might add files entries to package_list with signatures
        signer.sign(package_list)

    def _upload(self, package_list, remote):
        app = ConanApp(self._conan_api)
        app.remote_manager.check_credentials(remote)
        executor = UploadExecutor(app)
        executor.upload(package_list, remote)

    def upload_full(self, package_list: PackagesList, remote: Remote, enabled_remotes: List[Remote],
                    check_integrity=False, force=False, metadata: List[str] = None, dry_run=False):
        """ Does the whole process of uploading, including the possibility of parallelizing
        per recipe based on the ``core.upload:parallel`` conf.

        The steps that this method performs are:
            - calls ``conan_api.cache.check_integrity`` to ensure the packages are not corrupted
            - checks the upload policy of the recipes
                - (if it is ``"skip"``, it will not upload the binaries, but will still upload
                  the metadata)
            - checks which revisions already exist in the server so that it can skip the upload
            - prepares the artifacts to upload (compresses the conan_package.tgz)
            - executes the actual upload
            - uploads associated sources backups if any

        :param package_list: A PackagesList object with the recipes and packages to upload.
        :param remote: The remote to upload the packages to.
        :param enabled_remotes: A list of remotes that are enabled in the client.
            Recipe sources will attempt to be fetched from these remotes,
            and to possibly load python_requires from the listed recipes if necessary.
        :param check_integrity: If ``True``, it will check the integrity of the cache packages
            before uploading them. This is useful to ensure that the packages are not corrupted.
        :param force: If ``True``, it will force the upload of the recipes and packages,
            even if they already exist in the remote. Note that this might update the timestamps
        :param metadata: A list of patterns of metadata that should be uploaded.
            Default ``None`` means all metadata will be uploaded together with the package artifacts.
            If metadata contains an empty string (``""``),
            it means that no metadata files should be uploaded.
        :param dry_run: If ``True``, it will not perform the actual upload,
            but will still prepare the artifacts and check the upstream.
        """

        def _upload_pkglist(pkglist, subtitle=lambda _: None):
            if check_integrity:
                subtitle("Checking integrity of cache packages")
                self._conan_api.cache.check_integrity(pkglist)
            # Check if the recipes/packages are in the remote
            subtitle("Checking server for existing packages")
            self.check_upstream(pkglist, remote, enabled_remotes, force)
            subtitle("Preparing artifacts for upload")
            self.prepare(pkglist, enabled_remotes, metadata)

            if not dry_run:
                subtitle("Uploading artifacts")
                self._upload(pkglist, remote)
                backup_files = self._conan_api.cache.get_backup_sources(pkglist)
                self.upload_backup_sources(backup_files)

        t = time.time()
        ConanOutput().title(f"Uploading to remote {remote.name}")
        parallel = self._conan_api.config.get("core.upload:parallel", default=1, check_type=int)
        thread_pool = ThreadPool(parallel) if parallel > 1 else None
        if not thread_pool or len(package_list._data) <= 1:  # FIXME: Iteration when multiple rrevs
            _upload_pkglist(package_list, subtitle=ConanOutput().subtitle)
        else:
            ConanOutput().subtitle(f"Uploading with {parallel} parallel threads")
            thread_pool.map(_upload_pkglist, package_list.split())
        if thread_pool:
            thread_pool.close()
            thread_pool.join()
        elapsed = time.time() - t
        ConanOutput().success(f"Upload completed in {int(elapsed)}s\n")
        add_urls(package_list, remote)

    def upload_backup_sources(self, files: List) -> None:
        """
        Upload to the server the backup sources files, that have been typically gathered by
        CacheAPI.get_backup_sources()

        :param files: The list of files that must be uploaded
        """
        config = self._api_helpers.global_conf
        url = config.get("core.sources:upload_url", check_type=str)
        if url is None:
            return
        url = url if url.endswith("/") else url + "/"

        output = ConanOutput()
        output.subtitle("Uploading backup sources")
        if not files:
            output.info("No backup sources files to upload")
            return

        requester = self._api_helpers.requester
        uploader = FileUploader(requester, verify=True, config=config, source_credentials=True)
        # TODO: For Artifactory, we can list all files once and check from there instead
        #  of 1 request per file, but this is more general
        for file in files:
            basename = os.path.basename(file)
            full_url = url + basename
            is_summary = file.endswith(".json")
            file_kind = "summary" if is_summary else "file"
            try:
                if is_summary or not uploader.exists(full_url, auth=None):
                    output.info(f"Uploading {file_kind} '{basename}' to backup sources server")
                    uploader.upload(full_url, file, dedup=False, auth=None)
                else:
                    output.info(f"File '{basename}' already in backup sources server, "
                                "skipping upload")
            except (AuthenticationException, ForbiddenException) as e:
                if is_summary:
                    output.warning(f"Could not update summary '{basename}' in backup sources server. "
                                   "Skipping updating file but continuing with upload. "
                                   f"Missing permissions?: {e}")
                else:
                    raise ConanException(f"The source backup server '{url}' needs authentication"
                                         f"/permissions, please provide 'source_credentials.json': {e}")

        output.success("Upload backup sources complete\n")



import json
import os
import platform

import pytest
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import load


def test_empty_package():
    # Checks default values generated by conan for cpp_info
    client = TestClient()
    client.save({'conanfile.py': GenConanfile("mylib", "0.1")})
    client.run('create .')
    client.run('install --requires=mylib/0.1@ -g QbsDeps')

    module_path = os.path.join(client.current_folder, 'conan-qbs-deps', 'mylib.json')
    module_content = json.loads(load(module_path))

    assert module_content.get('package_name') == 'mylib'
    assert module_content.get('version') == '0.1'

    package_dir = module_content['package_dir']
    cpp_info = module_content['cpp_info']
    package_dir = package_dir.replace("/", "\\") if platform.system() == "Windows" else package_dir
    assert cpp_info.get('includedirs') == [os.path.join(package_dir, 'include')]
    assert cpp_info.get('libdirs') == [os.path.join(package_dir, 'lib')]
    assert cpp_info.get('bindirs') == [os.path.join(package_dir, 'bin')]
    assert cpp_info.get('libs') == []
    assert cpp_info.get('frameworkdirs') == []
    assert cpp_info.get('frameworks') == []
    assert cpp_info.get('defines') == []
    assert cpp_info.get('cflags') == []
    assert cpp_info.get('cxxflags') == []

    assert module_content.get('settings') == {}
    assert module_content.get('options') == {}


def test_empty_dirs():
    # Checks that we can override default values with empty directories
    conanfile = textwrap.dedent('''
        from conan import ConanFile

        class Recipe(ConanFile):
            name = 'mylib'
            version = '0.1'

            def package_info(self):
                self.cpp_info.includedirs = []
                self.cpp_info.libdirs = []
                self.cpp_info.bindirs = []
                self.cpp_info.libs = []
                self.cpp_info.frameworkdirs = []
        ''')
    client = TestClient()
    client.save({'conanfile.py': conanfile})
    client.run('create .')
    client.run('install --requires=mylib/0.1@ -g QbsDeps')

    module_path = os.path.join(client.current_folder, 'conan-qbs-deps', 'mylib.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert module_content.get('package_name') == 'mylib'
    assert module_content.get('version') == '0.1'

    assert 'cpp_info' in module_content
    cpp_info = module_content['cpp_info']
    assert cpp_info.get('includedirs') == []
    assert cpp_info.get('libdirs') == []
    assert cpp_info.get('bindirs') == []
    assert cpp_info.get('libs') == []
    assert cpp_info.get('frameworkdirs') == []
    assert cpp_info.get('frameworks') == []
    assert cpp_info.get('defines') == []
    assert cpp_info.get('cflags') == []
    assert cpp_info.get('cxxflags') == []

    assert module_content.get('settings') == {}
    assert module_content.get('options') == {}


def test_pkg_config_name():
    # Checks we can override module name using the "pkg_config_name" property
    conanfile = textwrap.dedent('''
        from conan import ConanFile

        class Recipe(ConanFile):
            name = 'mylib'
            version = '0.1'
            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "myfirstlib")
        ''')
    client = TestClient()
    client.save({'conanfile.py': conanfile})
    client.run('create .')
    client.run('install --requires=mylib/0.1@ -g QbsDeps')

    module_path = os.path.join(client.current_folder, 'conan-qbs-deps', 'myfirstlib.json')
    assert os.path.exists(module_path)


@pytest.mark.parametrize('host_os, arch, build_type', [
    ('Linux', 'x86_64', 'Debug'),
    ('Linux', 'x86_64', 'Release'),
    ('Linux', 'armv8', 'Debug'),
    ('Windows', 'x86_64', 'Debug'),
    ('Macos', 'armv8', 'Release'),
])
def test_settings(host_os, arch, build_type):

    conanfile = textwrap.dedent('''
    from conan import ConanFile
    class Recipe(ConanFile):
        settings = "os", "arch", "build_type"
        name = "mylib"
        version = "0.1"
    ''')

    client = TestClient()
    client.save({'conanfile.py': conanfile})
    common_cmd = '-s:h os={os} -s:h arch={arch} -s:h build_type={build_type}'.format(
        os=host_os, arch=arch, build_type=build_type
    )
    client.run('create . ' + common_cmd)
    client.run('install --requires=mylib/0.1@ -g QbsDeps ' + common_cmd)

    module_path = os.path.join(client.current_folder, 'conan-qbs-deps', 'mylib.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert 'settings' in module_content
    # Qbs only cares about os and arch (and maybe build type)
    assert module_content['settings'].get('os') == host_os
    assert module_content['settings'].get('arch') == arch
    assert module_content['settings'].get('build_type') == build_type


@pytest.mark.parametrize('shared', ['False', 'True'])
def test_options(shared):
    conanfile = textwrap.dedent('''
    from conan import ConanFile
    class Recipe(ConanFile):
        options = {"shared": [True, False]}
        default_options = {"shared": False}
        name = 'mylib'
        version = '0.1'
    ''')

    client = TestClient()
    client.save({'conanfile.py': conanfile})
    common_cmd = '-o:h shared={shared}'.format(shared=shared)
    client.run('create . ' + common_cmd)
    client.run('install --requires=mylib/0.1@ -g QbsDeps ' + common_cmd)

    module_path = os.path.join(client.current_folder, 'conan-qbs-deps', 'mylib.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert 'options' in module_content
    assert module_content['options'].get('shared') == shared


def test_components():
    """
        Checks a package with multiple components.
        Here we test component name and version override as well as the defaults.
    """
    conanfile = textwrap.dedent('''
    from conan import ConanFile

    class Recipe(ConanFile):

        name = 'mylib'
        version = '0.1'
        def package_info(self):
            self.cpp_info.components["mycomponent1"].set_property("pkg_config_name",
                                                                  "mycomponent")
            self.cpp_info.components["mycomponent2"].set_property("component_version",
                                                                  "19.8.199")
    ''')

    client = TestClient()
    client.save({'conanfile.py': conanfile})
    client.run('create .')
    client.run('install --requires=mylib/0.1@ -g QbsDeps')

    module1_path = os.path.join(client.current_folder, 'conan-qbs-deps', 'mycomponent.json')
    assert os.path.exists(module1_path)
    module1_content = json.loads(load(module1_path))

    assert module1_content.get('package_name') == 'mylib'
    assert module1_content.get('version') == '0.1'
    assert module1_content.get('dependencies') == []

    module2_path = os.path.join(client.current_folder, 'conan-qbs-deps', 'mycomponent2.json')
    assert os.path.exists(module2_path)
    module2_content = json.loads(load(module2_path))

    assert module2_content.get('package_name') == 'mylib'
    assert module2_content.get('version') == '19.8.199'
    assert module2_content.get('dependencies') == []

    main_module_path = os.path.join(client.current_folder, 'conan-qbs-deps', 'mylib.json')
    assert os.path.exists(main_module_path)
    main_module_content = json.loads(load(main_module_path))

    assert main_module_content.get('package_name') == 'mylib'
    assert main_module_content.get('version') == '0.1'
    assert main_module_content.get('dependencies') == [
        {"name": "mycomponent", "version": "0.1"},  # name overriden, version default
        {"name": "mycomponent2", "version": "19.8.199"}  # name default, version overriden
    ]


def test_cpp_info_requires():
    """
    Testing a complex structure like:

    * first/0.1
        - Global pkg_config_name == "myfirstlib"
        - Components: "cmp1"
    * other/0.1
    * second/0.2
        - Requires: "first/0.1"
        - Components: "mycomponent", "myfirstcomp"
            + "mycomponent" requires "first::cmp1"
            + "myfirstcomp" requires "mycomponent"
    * third/0.4
        - Requires: "second/0.2", "other/0.1"

    Expected file structure after running QbsDeps as generator:
        - other.json
        - myfirstlib-cmp1.json
        - myfirstlib.json
        - second-mycomponent.json
        - second-myfirstcomp.json
        - second.json
        - third.json
    """

    client = TestClient()
    # first
    conanfile = textwrap.dedent('''
        from conan import ConanFile

        class Recipe(ConanFile):
            name = "first"
            version = "0.1"
            def package_info(self):
                self.cpp_info.set_property("pkg_config_name", "myfirstlib")
                self.cpp_info.components["cmp1"].libs = ["libcmp1"]
    ''')
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    # other
    client.save({"conanfile.py": GenConanfile("other", "0.1")}, clean_first=True)
    client.run("create .")

    # second
    conanfile = textwrap.dedent('''
    from conan import ConanFile

    class PkgConfigConan(ConanFile):
        name = "second"
        version = "0.2"
        requires = "first/0.1"

        def package_info(self):
            self.cpp_info.components["mycomponent"].requires.append("first::cmp1")
            self.cpp_info.components["myfirstcomp"].requires.append("mycomponent")
    ''')
    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run("create .")

    # third
    client.save({"conanfile.py": GenConanfile("third", "0.3").with_require("second/0.2")
                                                             .with_require("other/0.1")},
                clean_first=True)
    client.run("create .")

    client2 = TestClient(cache_folder=client.cache_folder)
    conanfile = textwrap.dedent("""
        [requires]
        third/0.3
        other/0.1

        [generators]
        QbsDeps
    """)
    client2.save({"conanfile.txt": conanfile})
    client2.run("install .")

    # first
    module_path = os.path.join(client2.current_folder, 'conan-qbs-deps', 'cmp1.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert module_content.get('package_name') == 'first'
    assert module_content.get('version') == '0.1'
    assert module_content.get('dependencies') == []

    module_path = os.path.join(client2.current_folder, 'conan-qbs-deps', 'myfirstlib.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert module_content.get('package_name') == 'first'
    assert module_content.get('version') == '0.1'
    assert module_content.get('dependencies') == [{'name': 'cmp1', 'version': '0.1'}]

    # other
    module_path = os.path.join(client2.current_folder, 'conan-qbs-deps', 'other.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert module_content.get('package_name') == 'other'
    assert module_content.get('version') == '0.1'
    assert module_content.get('dependencies') == []

    # second.mycomponent
    module_path = os.path.join(client2.current_folder, 'conan-qbs-deps', 'mycomponent.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert module_content.get('package_name') == 'second'
    assert module_content.get('version') == '0.2'
    assert module_content.get('dependencies') == [{'name': 'cmp1', 'version': '0.1'}]

    # second.myfirstcomp
    module_path = os.path.join(client2.current_folder, 'conan-qbs-deps', 'myfirstcomp.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert module_content.get('package_name') == 'second'
    assert module_content.get('version') == '0.2'
    assert module_content.get('dependencies') == [{'name': 'mycomponent', 'version': '0.2'}]

    # second
    module_path = os.path.join(client2.current_folder, 'conan-qbs-deps', 'second.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert module_content.get('package_name') == 'second'
    assert module_content.get('version') == '0.2'
    assert module_content.get('dependencies') == [
        {'name': 'mycomponent', 'version': '0.2'},
        {'name': 'myfirstcomp', 'version': '0.2'}
    ]

    # third
    module_path = os.path.join(client2.current_folder, 'conan-qbs-deps', 'third.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert module_content.get('package_name') == 'third'
    assert module_content.get('version') == '0.3'
    assert module_content.get('dependencies') == [
        {'name': 'second', 'version': '0.2'},
        {'name': 'other', 'version': '0.1'}
    ]


# see https://github.com/conan-io/conan/issues/10341
def test_components_conflict():
    """ If component has the same name as the root package, skip root package
    """
    conanfile = textwrap.dedent('''
    from conan import ConanFile

    class Recipe(ConanFile):

        name = 'mylib'
        version = '0.1'
        def package_info(self):
            self.cpp_info.set_property("pkg_config_name", "mycoollib")
            self.cpp_info.components["_mycoollib"].defines = ["MAGIC_DEFINE"]
            self.cpp_info.components["_mycoollib"].set_property("pkg_config_name",
                                                               "mycoollib")
    ''')

    client = TestClient()
    client.save({'conanfile.py': conanfile})
    client.run('create .')
    client.run('install --requires=mylib/0.1@ -g QbsDeps')

    module_path = os.path.join(client.current_folder, 'conan-qbs-deps', 'mycoollib.json')
    assert os.path.exists(module_path)
    module_content = json.loads(load(module_path))

    assert module_content.get('package_name') == 'mylib'
    assert module_content.get('version') == '0.1'
    assert 'cpp_info' in module_content
    cpp_info = module_content['cpp_info']
    assert cpp_info.get('defines') == ["MAGIC_DEFINE"]



import os
import textwrap

from conan.api.output import ConanOutput
from conan.internal.default_settings import migrate_settings_file
from conans.migrations import Migrator
from conan.internal.util.files import load, save

CONAN_GENERATED_COMMENT = "This file was generated by Conan"


def update_file(file_path, new_content):
    """
    Update any file path given with the new content.
    Notice that the file is only updated whether it contains the ``CONAN_GENERATED_COMMENT``.

    :param file_path: ``str`` path to the file.
    :param new_content: ``str`` content to be saved.
    """
    out = ConanOutput()
    file_name = os.path.basename(file_path)

    if not os.path.exists(file_path):
        save(file_path, new_content)
    else:
        content = load(file_path)

        first_line = content.lstrip().split("\n", 1)[0]

        if CONAN_GENERATED_COMMENT in first_line and content != new_content:
            save(file_path, new_content)
            out.success(f"Migration: Successfully updated {file_name}")


class ClientMigrator(Migrator):

    def __init__(self, cache_folder, current_version):
        self.cache_folder = cache_folder
        super(ClientMigrator, self).__init__(cache_folder, current_version)

    def _apply_migrations(self, old_version):
        # Migrate the settings if they were the default for that version
        # Time for migrations!
        # Update settings.yml
        migrate_settings_file(self.cache_folder)
        # Update compatibility.py, app_compat.py, and cppstd_compat.py.
        from conan.internal.graph.compatibility import migrate_compatibility_files
        migrate_compatibility_files(self.cache_folder)
        # Update profile plugin
        from conan.internal.api.profile.profile_loader import migrate_profile_plugin
        migrate_profile_plugin(self.cache_folder)

        # let the back migration files be stored
        # if there was not a previous install (old_version==None)
        if old_version is None or old_version < "2.4":
            _migrate_default_compatibility(self.cache_folder)


def _migrate_default_compatibility(cache_folder):
    # just the back migration
    undo = textwrap.dedent("""\
        import os

        def migrate(home_folder):
            from conans.client.graph.compatibility import migrate_compatibility_files
            migrate_compatibility_files(home_folder)
        """)
    path = os.path.join(cache_folder, "migrations", "2.4_1-migrate.py")
    save(path, undo)



import os
import platform
import textwrap

import pytest

from conan.test.assets.cmake import gen_cmakelists
from conan.test.assets.sources import gen_function_cpp
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.mocks import ConanFileMock
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.tools.env.environment import environment_wrap_command


@pytest.fixture
def client():
    # We use special characters and spaces, to check everything works
    # https://github.com/conan-io/conan/issues/12648
    # FIXME: This path still fails the creation of the deactivation script
    cache_folder = os.path.join(temp_folder(), "[sub] folder")
    client = TestClient(cache_folder)
    conanfile = str(GenConanfile("pkg", "0.1"))
    conanfile += """

    def package_info(self):
        self.buildenv_info.define_path("MYPATH1", "c:/path/to/ar")
        self.runenv_info.define("MYVAR1", 'some nice content\" with quotes')
    """
    client.save({"conanfile.py": conanfile})
    client.run("create .")
    client.save_home({"global.conf": "tools.env.virtualenv:powershell=powershell.exe\n"})
    return client


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows powershell")
def test_virtualenv(client):
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile

        class ConanFileToolsTest(ConanFile):
            name = "app"
            version = "0.1"
            requires = "pkg/0.1"
            apply_env = False

            def build(self):
                self.output.info("----------BUILD----------------")
                self.run("set")
                self.output.info("----------RUN----------------")
                self.run("set", env="conanrun")
        """)
    client.save({"conanfile.py": conanfile})
    client.run("install . -s:b os=Windows -s:h os=Windows")

    assert not os.path.exists(os.path.join(client.current_folder, "conanbuildenv.sh"))
    assert not os.path.exists(os.path.join(client.current_folder, "conanbuildenv.bat"))
    assert not os.path.exists(os.path.join(client.current_folder, "conanrunenv.sh"))
    assert not os.path.exists(os.path.join(client.current_folder, "conanrunenv.bat"))
    with open(os.path.join(client.current_folder, "conanbuildenv.ps1"), "r", encoding="utf-16") as f:
        buildenv = f.read()
    assert '$env:MYPATH1="c:/path/to/ar"' in buildenv
    build = client.load("conanbuild.ps1")
    assert "conanbuildenv.ps1" in build

    with open(os.path.join(client.current_folder, "conanrunenv.ps1"), "r", encoding="utf-16") as f:
        run_contents = f.read()
    assert '$env:MYVAR1="some nice content`" with quotes"' in run_contents

    client.run("create .")
    assert "MYPATH1=c:/path/to/ar" in client.out
    assert 'MYVAR1=some nice content" with quotes' in client.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows powershell")
@pytest.mark.parametrize("powershell", [True, "powershell.exe", "pwsh"])
def test_virtualenv_test_package(powershell):
    """ The test_package could crash if not cleaning correctly the test_package
    output folder. This will still crassh if the layout is not creating different build folders
    https://github.com/conan-io/conan/issues/12764
    """
    client = TestClient()
    test_package = textwrap.dedent(r"""
        from conan import ConanFile
        from conan.tools.files import save
        class Test(ConanFile):
            def requirements(self):
                self.requires(self.tested_reference_str)
            def generate(self):
                # Emulates vcvars.bat behavior
                save(self, "myenv.bat", "echo MYENV!!!\nset MYVC_CUSTOMVAR1=PATATA1")
                self.env_scripts.setdefault("build", []).append("myenv.bat")
                save(self, "myps.ps1", "echo MYPS1!!!!\n$env:MYVC_CUSTOMVAR2=\"PATATA2\"")
                self.env_scripts.setdefault("build", []).append("myps.ps1")
            def layout(self):
                self.folders.build = "mybuild"
                self.folders.generators = "mybuild"
            def test(self):
                self.run('mkdir "hello world"')
                self.run("dir")
                self.run('cd "hello world"')
                self.run("set MYVC_CUSTOMVAR1")
                self.run("set MYVC_CUSTOMVAR2")
            """)
    client.save({"conanfile.py": GenConanfile("pkg", "1.0"),
                 "test_package/conanfile.py": test_package})
    client.run("create .")
    assert "hello world" in client.out
    assert "MYENV!!!" in client.out
    assert "MYPS1!!!!" in client.out
    assert "MYVC_CUSTOMVAR1=PATATA1" in client.out
    assert "MYVC_CUSTOMVAR2=PATATA2" in client.out
    # This was crashing because the .ps1 of test_package was not being cleaned
    client.run(f"create . -c tools.env.virtualenv:powershell={powershell}")
    assert "hello world" in client.out
    assert "MYENV!!!" in client.out
    assert "MYPS1!!!!" in client.out
    assert "MYVC_CUSTOMVAR1=PATATA1" in client.out
    assert "MYVC_CUSTOMVAR2=PATATA2" in client.out

@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows powershell")
@pytest.mark.parametrize("powershell", [True, "powershell.exe", "pwsh"])
def test_vcvars(powershell):
    client = TestClient()
    conanfile = textwrap.dedent(r"""
        from conan import ConanFile
        from conan.tools.cmake import CMake, cmake_layout, CMakeToolchain, CMakeDeps
        from conan.tools.env import VirtualBuildEnv

        class Conan(ConanFile):
           settings = "os", "compiler", "build_type", "arch"

           generators = 'CMakeDeps', 'CMakeToolchain'

           def layout(self):
              cmake_layout(self)

           def build(self):
              cmake = CMake(self)
              cmake.configure()
              cmake.build()
    """)
    hello_cpp = gen_function_cpp(name="main")
    cmakelists = gen_cmakelists(appname="hello", appsources=["hello.cpp"])
    client.save({"conanfile.py": conanfile, "hello.cpp": hello_cpp, "CMakeLists.txt": cmakelists})
    powershell_exe = "powershell.exe" if powershell == "powershell" else "pwsh"
    client.run(f"build . -c tools.env.virtualenv:powershell={powershell} -c tools.cmake.cmaketoolchain:generator=Ninja")
    client.run_command(rf'{powershell_exe} -Command ".\build\Release\generators\conanbuild.ps1; dir env:"')
    #check the conanbuid.ps1 activation message
    assert "conanvcvars.ps1: Activated environment" in client.out
    #check that the new env variables are set
    assert "VSCMD_ARG_VCVARS_VER" in client.out

    client.run_command(rf'{powershell_exe} -Command ".\build\Release\generators\conanvcvars.ps1"')
    assert client.out.strip() == "conanvcvars.ps1: Activated environment"

    conanbuild = client.load(r".\build\Release\generators\conanbuild.ps1")
    vcvars_ps1 = client.load(r".\build\Release\generators\conanvcvars.ps1")
    #check that the conanvcvars.ps1 is being added to the conanbuild.ps1
    assert "conanvcvars.ps1" in conanbuild
    #check that the conanvcvars.ps1 is setting the environment
    assert "conanvcvars.bat&set" in vcvars_ps1



@pytest.mark.skipif(platform.system() != "Windows", reason="Test for powershell")
@pytest.mark.parametrize("powershell", [True, "powershell.exe", "pwsh", "powershell.exe -NoProfile", "pwsh -NoProfile"])
def test_concatenate_build_and_run_env(powershell):
    # this tests that if we have both build and run env, they are concatenated correctly when using
    # powershell
    client = TestClient(path_with_spaces=True)
    compiler_bat = "@echo off\necho MYTOOL {}!!\n"
    conanfile = textwrap.dedent("""\
        import os
        from conan import ConanFile
        from conan.tools.files import copy
        class Pkg(ConanFile):
            exports_sources = "*"
            package_type = "application"
            def package(self):
                copy(self, "*", self.build_folder, os.path.join(self.package_folder, "bin"))
        """)

    num_deps = 2
    for i in range(num_deps):
        client.save({"conanfile.py": conanfile,
                     "mycompiler{}.bat".format(i): compiler_bat.format(i)})
        client.run("create . --name=pkg{} --version=0.1".format(i))

    conanfile = textwrap.dedent("""\
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os"
            tool_requires = "pkg0/0.1"
            requires = "pkg1/0.1"
            generators = "VirtualBuildEnv", "VirtualRunEnv"
        """)

    client.save({"conanfile.py": conanfile}, clean_first=True)
    client.run(f'install . -c tools.env.virtualenv:powershell="{powershell}"')
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.env.virtualenv:powershell", powershell)
    cmd = environment_wrap_command(conanfile,["conanrunenv", "conanbuildenv"],
                                   client.current_folder,"mycompiler0.bat")
    client.run_command(cmd)
    assert "MYTOOL 0!!" in client.out

    cmd = environment_wrap_command(conanfile,["conanrunenv", "conanbuildenv"],
                                   client.current_folder,"mycompiler1.bat")
    client.run_command(cmd)
    assert "MYTOOL 1!!" in client.out


@pytest.mark.parametrize("powershell", [None, True, False])
def test_powershell_deprecated_message(powershell):
    client = TestClient(light=True)
    conanfile = textwrap.dedent("""\
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os"
            name = "pkg"
            version = "0.1"
        """)

    client.save({"conanfile.py": conanfile})
    powershell_arg = f'-c tools.env.virtualenv:powershell={powershell}' if powershell is not None else ""
    client.run(f'install . {powershell_arg}')
    # only show message if the value is set to False or True if not set do not show message
    if powershell is not None:
        assert "Boolean values for 'tools.env.virtualenv:powershell' are deprecated" in client.out
    else:
        assert "Boolean values for 'tools.env.virtualenv:powershell' are deprecated" not in client.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Test for powershell")
@pytest.mark.parametrize("powershell", [True, "pwsh", "powershell.exe"])
def test_powershell_quoting(powershell):
    client = TestClient(path_with_spaces=False)
    conanfile = textwrap.dedent("""\
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os"
            name = "pkg"
            version = "0.1"
            def build(self):
                self.run('python -c "print(\\'Hello World\\')"', scope="build")
        """)

    client.save({"conanfile.py": conanfile})
    client.run(f'create . -c tools.env.virtualenv:powershell={powershell}')
    assert "Hello World" in client.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
def test_verbosity_flag():
    tc = TestClient()
    tc.run("new cmake_lib -d name=pkg -d version=1.0")
    tc.run('create . -tf="" -c tools.build:verbosity=verbose '
           '-c tools.env.virtualenv:powershell=powershell.exe')

    assert "/verbosity:Detailed" in tc.out
    assert "-verbosity:Detailed" not in tc.out
